import os
import argparse
from Bio import SeqIO


get_arCOG_seq_usage = '''
=========================== get_arCOG_seq example commands ===========================

TreeSAK get_arCOG_seq -id cog_id.txt -db_dir /Users/songweizhi/DB/arCOG18 -o op_dir

# required db files
ar18.ar14.02.csv, arCOG_names_220807.txt and ar18.fa

======================================================================================
'''


def get_arCOG_seq(args):

    cog_id_txt          = args['i']
    db_dir              = args['db_dir']
    op_dir              = args['o']
    force_create_dir    = args['f']

    ar18_ar14_02_csv = '%s/ar18.ar14.02.csv'        % db_dir
    cog_des_txt      = '%s/arCOG_names_220807.txt'  % db_dir
    ar18_fa          = '%s/ar18.fa'                 % db_dir
    cog_metadata_txt = '%s/metadata.txt'            % op_dir


    if os.path.isdir(op_dir) is True:
        if force_create_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder already exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    cog_des_dict = dict()
    for each_cog in open(cog_des_txt, encoding="ISO-8859-1"):
        each_cog_split = each_cog.strip().split('\t')
        cog_des_dict[each_cog_split[0]] = each_cog_split[1:]

    cog_id_set = set()
    for each_cog in open(cog_id_txt):
        cog_id_set.add(each_cog.strip().replace('ArCOG', 'arCOG'))

    seq_id_set = set()
    seq_to_arcog_dict = dict()
    arcog_to_seq_dict = dict()
    for each_line in open(ar18_ar14_02_csv):
        each_line_split = each_line.strip().split(',')
        arcog_id = each_line_split[6]
        seq_id = each_line_split[2]

        if arcog_id in cog_id_set:
            seq_id_set.add(seq_id)
            if arcog_id not in arcog_to_seq_dict:
                arcog_to_seq_dict[arcog_id] = {seq_id}
            else:
                arcog_to_seq_dict[arcog_id].add(seq_id)

            if seq_id not in seq_to_arcog_dict:
                seq_to_arcog_dict[seq_id] = {arcog_id}
            else:
                seq_to_arcog_dict[seq_id].add(arcog_id)

    # write out sequence by arCOG
    for each_seq in SeqIO.parse(ar18_fa, 'fasta'):
        seq_id = each_seq.id
        if seq_id in seq_id_set:
            seq_cog_set = seq_to_arcog_dict.get(seq_id, [])
            seq_cog_list = [i for i in seq_cog_set]
            if len(seq_cog_list) == 1:
                pwd_fa = '%s/%s.fa' % (op_dir, seq_cog_list[0])
                with open(pwd_fa, 'a') as pwd_fa_handle:
                    pwd_fa_handle.write('>%s\n' % seq_id)
                    pwd_fa_handle.write('%s\n' % str(each_seq.seq))

    # write out metadata
    cog_metadata_txt_handle = open(cog_metadata_txt, 'w')
    for each_c in sorted([i for i in cog_id_set]):
        each_c_desc = '\t'.join(cog_des_dict[each_c])
        cog_metadata_txt_handle.write('%s\t%s\n' % (each_c, each_c_desc))
    cog_metadata_txt_handle.close()


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i',      required=True,                       help='arCOD id file, one id per line')
    parser.add_argument('-db_dir', required=True,                       help='database folder')
    parser.add_argument('-o',      required=True,                       help='output folder')
    parser.add_argument('-f',      required=False, action="store_true", help='force overwrite existing output folder')
    args = vars(parser.parse_args())
    get_arCOG_seq(args)



import os
import argparse
import pandas as pd
import multiprocessing as mp
from statsmodels.stats.multitest import multipletests


PhyloBiAssoc_usage = '''
============================= PhyloBiAssoc example commands =============================

BioSAK PhyloBiAssoc -i demo.tre -d demo.txt -o op_dir -t 10 -f

# Note, header for the first two columns in -d has to be "ID" and "cate"!!!

# It will perform:
# 1) binaryPGLMM test if phylosig p-value <= 0.05 (significant phylogenetic signal)
# 2) chi-squared test if phylosig p-value > 0.05  (no phylogenetic signal)
# 3) do nothing if phylosig returns NaN (might due to the same value across all genomes)

# https://www.rdocumentation.org/packages/ape/versions/5.7-1/topics/binaryPGLMM

=========================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def subset_df(file_in, rows_to_keep, cols_to_keep, sep_symbol, row_name_pos, column_name_pos, file_out):

    df = pd.read_csv(file_in, sep=sep_symbol, header=column_name_pos, index_col=row_name_pos)

    if len(rows_to_keep) == 0:
        if len(cols_to_keep) == 0:
            subset_df = df.loc[:, :]
        else:
            subset_df = df.loc[:, cols_to_keep]
    else:
        if len(cols_to_keep) == 0:
            subset_df = df.loc[rows_to_keep, :]
        else:
            subset_df = df.loc[rows_to_keep, cols_to_keep]

    subset_df.to_csv(file_out, sep=sep_symbol)


def PhyloBiAssoc(args):

    tree_file           = args['i']
    data_file           = args['d']
    op_dir              = args['o']
    num_threads         = args['t']
    force_create_op_dir = args['f']

    pwd_current_script  = os.path.realpath(__file__)
    current_script_path = '/'.join(pwd_current_script.split('/')[:-1])
    PhyloBiAssoc_R      = '%s/PhyloBiAssoc.R' % current_script_path

    cmd_txt                     = '%s/cmds.txt'                         % op_dir
    df_subset_dir               = '%s/df_subset'                        % op_dir
    stats_op_dir                = '%s/stats_results'                    % op_dir
    combined_stats_txt          = '%s/stats_results_all.txt'            % op_dir
    combined_stats_txt_sig      = '%s/stats_results_0.05.txt'           % op_dir
    combined_stats_txt_adjusted = '%s/stats_results_0.05_adjusted.txt'  % op_dir

    # create op_dir
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output directory exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % df_subset_dir)
    os.system('mkdir %s' % stats_op_dir)

    # read in dataframe
    df = pd.read_csv(data_file, sep='\t', header=0, index_col=0)
    col_header_list = list(df.columns.values)

    subset_dict = dict()
    for each_col in col_header_list[1:]:
        subset_dict[each_col] = ['cate', each_col]

    # subset dataframe
    cmd_txt_handle = open(cmd_txt, 'w')
    stats_cmd_list = []
    op_stats_txt_set = set()
    for each_subset in subset_dict:
        cols_to_keep   = subset_dict[each_subset]
        df_subset_file = '%s/%s.tab' % (df_subset_dir, each_subset)
        stats_out_txt  = '%s/%s.txt' % (stats_op_dir, each_subset)
        subset_df(data_file, set(), cols_to_keep, '\t', 0, 0, df_subset_file)
        stats_cmd = 'Rscript %s -t %s -d %s > %s' % (PhyloBiAssoc_R, tree_file, df_subset_file, stats_out_txt)
        cmd_txt_handle.write(stats_cmd + '\n')
        stats_cmd_list.append(stats_cmd)
        op_stats_txt_set.add(stats_out_txt)
    cmd_txt_handle.close()

    print('Processing %s objects with %s cores' % (len(stats_cmd_list), num_threads))
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, stats_cmd_list)
    pool.close()
    pool.join()
    #os.system('cp /Users/songweizhi/Documents/Research/Sponge/12_PhyloBiAssoc_wd/PhyloBiAssoc_wd_backup/stats_results/*.txt %s/' % stats_op_dir)

    # combine stats results
    sig_list_id = []
    sig_list_value = []

    combined_stats_txt_handle = open(combined_stats_txt, 'w')
    combined_stats_txt_handle.write('ID	phylosig	binaryPGLMM	chisq.test	coefficient	significance\n')
    combined_stats_txt_sig_handle = open(combined_stats_txt_sig, 'w')
    for each_file in sorted(list(op_stats_txt_set)):
        f_path, f_base, f_ext = sep_path_basename_ext(each_file)
        for each_line in open(each_file):
            if not each_line.startswith('ID\tphylosig\tbinaryPGLMM\tchisq.test'):
                each_line_split = each_line.strip().split('\t')
                significance = each_line_split[5]
                combined_stats_txt_handle.write('%s\t%s\n' % (f_base, '\t'.join(each_line_split[1:])))
                if significance == 'y':
                    combined_stats_txt_sig_handle.write(f_base + '\n')
                    sig_bi = each_line_split[2]
                    sig_chi = each_line_split[3]
                    current_sig = ''
                    if sig_bi == 'na':
                        current_sig = sig_chi
                    elif sig_chi == 'na':
                        current_sig = sig_bi
                    sig_list_id.append(f_base)
                    sig_list_value.append(float(current_sig))
    combined_stats_txt_handle.close()
    combined_stats_txt_sig_handle.close()

    # perform Bonferroni correction
    sig_list_value_adjusted = list(multipletests(sig_list_value, alpha=0.1, method='bonferroni')[1])

    # write out adjusted p values
    combined_stats_txt_adjusted_handle = open(combined_stats_txt_adjusted, 'w')
    combined_stats_txt_adjusted_handle.write('ID\tadjusted_p_value\n')
    for (id, adjusted_p) in zip(sig_list_id, sig_list_value_adjusted):
        if adjusted_p <= 0.05:
            combined_stats_txt_adjusted_handle.write('%s\t%s\n' % (id, adjusted_p))
    combined_stats_txt_adjusted_handle.close()

    # Final report
    print('Results exported to: \n%s\n%s' % (combined_stats_txt, combined_stats_txt_adjusted))
    print('Done')


if __name__ == "__main__":

    PhyloBiAssoc_parser = argparse.ArgumentParser(usage=PhyloBiAssoc_usage)
    PhyloBiAssoc_parser.add_argument('-i', required=True,                       help='tree file')
    PhyloBiAssoc_parser.add_argument('-d', required=True,                       help='data file')
    PhyloBiAssoc_parser.add_argument('-o', required=True,                       help='output directory')
    PhyloBiAssoc_parser.add_argument('-t', required=False, type=int, default=1, help='number of threads, default: 1')
    PhyloBiAssoc_parser.add_argument('-f', required=False, action="store_true", help='force overwrite')
    args = vars(PhyloBiAssoc_parser.parse_args())
    PhyloBiAssoc(args)



import os
import argparse
from ete3 import Tree


LcaToLeaves_usage = '''
==================== LcaToLeaves example commands ====================

BioSAK LcaToLeaves -s species_tree.ale.stree -n 123
BioSAK LcaToLeaves -s species_tree.ale.stree -n internal_nodes.txt

======================================================================
'''


def lca_to_two_leaves(species_tree_from_ale, internal_node_id):

    # read in ale species tree
    stree_ale = Tree(species_tree_from_ale, format=1)

    # get all leaves of the internal node
    internal_node = stree_ale.search_nodes(name=internal_node_id)[0]
    internal_node_leaf_object = internal_node.get_leaves()
    internal_node_leaf_set = set()
    for each_leaf in internal_node_leaf_object:
        internal_node_leaf_set.add(each_leaf.name)

    # get the two leaves needed
    targeted_two_leaves = []
    leaves_found = False
    for leaf_1 in internal_node_leaf_set:
        for leaf_2 in internal_node_leaf_set:
            if leaf_1 != leaf_2:
                if leaves_found is False:
                    current_lca_id = stree_ale.get_common_ancestor(leaf_1, leaf_2).name
                    if current_lca_id == internal_node_id:
                        targeted_two_leaves.append(leaf_1)
                        targeted_two_leaves.append(leaf_2)
                        leaves_found = True

    return targeted_two_leaves[0], targeted_two_leaves[1]


def LcaToLeaves(args):

    species_tree_from_ale = args['s']
    internal_node         = args['n']

    internal_node_set = set()
    if os.path.isfile(internal_node) is False:
        internal_node_set.add(internal_node)
    else:
        for each_node in open(internal_node):
            internal_node_set.add(each_node.strip())

    for each_internal_node in internal_node_set:
        leaf_1, leaf_2 = lca_to_two_leaves(species_tree_from_ale, each_internal_node)
        print('%s\t%s\t%s' % (each_internal_node, leaf_1, leaf_2))

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-s',   required=True,   help='the .stree file from ALE')
    parser.add_argument('-n',   required=True,   help='internal node(s)')
    args = vars(parser.parse_args())
    LcaToLeaves(args)



import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


def PlotMCMC(num_list_x, num_list_y, err_range_x, err_range_y, pwd_figure):

    x_err_l = []
    x_err_r = []
    y_err_l = []
    y_err_u = []
    n = 0
    while n < len(num_list_x):
        x_value = num_list_x[n]
        y_value = num_list_y[n]
        x_range = err_range_x[n]
        y_range = err_range_y[n]
        x_l_dist = abs(x_value - x_range[0])
        x_r_dist = abs(x_range[1] - x_value)
        y_l_dist = abs(y_value - y_range[0])
        y_u_dist = abs(y_range[1] - y_value)
        x_err_l.append(x_l_dist)
        x_err_r.append(x_r_dist)
        y_err_l.append(y_l_dist)
        y_err_u.append(y_u_dist)
        n += 1

    plt.scatter(num_list_x, num_list_y, s=0)
    plt.errorbar(num_list_x, num_list_y, xerr=[x_err_l, x_err_r], yerr=[y_err_l, y_err_u], ls='none', ecolor='black', elinewidth=1)
    plt.tight_layout()
    plt.savefig(pwd_figure)
    plt.close()


num_list_x = [1, 2, 3, 4, 5]
num_list_y = [1, 2, 3, 4, 5]
err_range_x = [[0.8, 1.1], [1.5, 2.1], [2.9, 3.7], [3.2, 4.1], [4.5, 5.5]]
err_range_y = [[0.7, 1.3], [1.9, 2.5], [2.9, 3.7], [3.1, 4.4], [4.5, 5.5]]
pwd_figure = '/Users/songweizhi/Desktop/aaa.png'

PlotMCMC(num_list_x, num_list_y, err_range_x, err_range_y, pwd_figure)



def gene_gain_and_loss(TableEvents_tsv):

    for each_line in open(TableEvents_tsv):
        if not each_line.startswith('Family\tBranchType\tBranch'):
            each_line_split = each_line.strip().split('\t')
            protein_family_id   = each_line_split[0]
            node_id             = each_line_split[2]
            Losses              = float(each_line_split[5])
            Originations        = float(each_line_split[6])
            Extinctinonprob     = float(each_line_split[9])

            print('%s\t%s\t%s\t%s\t%s' % (protein_family_id, node_id, Losses, '', Extinctinonprob))


TableEvents_tsv = '/Users/songweizhi/Desktop/Sponge_r220/10_dRep95_255_ALE_wd/dRep95_255_ALE3_op_dir_80/TableEvents.tsv'

gene_gain_and_loss(TableEvents_tsv)




import os
import argparse
from Bio import AlignIO


SingleLinePhy_usage = '''
======== SingleLinePhy example commands ========

TreeSAK SingleLinePhy -i in.phy -o out.phy

================================================
'''


def SingleLinePhy(args):

    phy_in  = args['i']
    phy_out = args['o']

    # check input file
    if os.path.isfile(phy_in) is False:
        print('input file not found, program exited!')
        exit()

    alignment = AlignIO.read(phy_in, 'phylip-relaxed')

    max_seq_id_len = 0
    for each_seq in alignment:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phy_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
        for each_seq in alignment:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))

    print('Done!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i',   required=True,                          help='input file')
    parser.add_argument('-o',   required=True,                          help='output file')
    args = vars(parser.parse_args())
    SingleLinePhy(args)



import os
import argparse


BMGE_usage = '''
======================= BMGE example commands =======================

# require: java

TreeSAK BMGE -p demo -i input.aln -m BLOSUM30 -esc 0.55

# Settings for calculating split score (Nina Dombrowski): 
# -t AA -m BLOSUM30 -h 0.55

=====================================================================
'''


def BMGE(args):

    op_prefix               = args['p']
    msa_in                  = args['i']
    trim_model              = args['m']
    entropy_score_cutoff    = args['esc']

    # define file name
    msa_out_fasta  = '%s.BMGE.fasta'  % op_prefix

    # specify path to BMGE.jar
    current_file_path   = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar        = '%s/BMGE.jar' % current_file_path

    # run BMGE
    bmge_cmd = 'java -jar %s -i %s -m %s -t AA -h %s -of %s' % (pwd_bmge_jar, msa_in, trim_model, entropy_score_cutoff, msa_out_fasta)
    print(bmge_cmd)
    os.system(bmge_cmd)

    print('Done!')


if __name__ == '__main__':

    BMGE_parser = argparse.ArgumentParser()
    BMGE_parser.add_argument('-p',   required=True,                         help='output prefix')
    BMGE_parser.add_argument('-i',   required=True,                         help='input MSA')
    BMGE_parser.add_argument('-m',   required=False, default='BLOSUM30',    help='trim model, default: BLOSUM30')
    BMGE_parser.add_argument('-esc', required=False, default='0.55',        help='entropy score cutoff, default: 0.55')
    args = vars(BMGE_parser.parse_args())
    BMGE(args)



import os
import glob
import math
import random
import argparse
import seaborn as sns
from ete3 import Tree
from itolapi import Itol
from PyPDF3.pdf import PageObject
from PyPDF3 import PdfFileWriter, PdfFileReader
from PyPDF3.generic import RectangleObject
# https://pypdf2.readthedocs.io/en/3.0.0/user/adding-pdf-annotations.html

def merge_pdf(pdf_1, pdf_2, margin_size, op_pdf):

    page1 = PdfFileReader(open(pdf_1, "rb"), strict=False).getPage(0)
    page2 = PdfFileReader(open(pdf_2, "rb"), strict=False).getPage(0)

    total_width  = page1.mediaBox.upperRight[0] + page2.mediaBox.upperRight[0] + margin_size*3
    total_height = max([page1.mediaBox.upperRight[1], page2.mediaBox.upperRight[1]]) + margin_size*2
    new_page = PageObject.createBlankPage(None, total_width, total_height)
    new_page.mergeTranslatedPage(page1, margin_size, (total_height-margin_size-page1.mediaBox.upperRight[1]))
    new_page.mergeTranslatedPage(page2, (page1.mediaBox.upperRight[0] + margin_size*2), margin_size)
    output = PdfFileWriter()
    output.addPage(new_page)
    output.write(open(op_pdf, "wb"))


merge_pdf('/Users/songweizhi/Desktop/1.pdf',
          '/Users/songweizhi/Desktop/2.pdf',
          66,
          '/Users/songweizhi/Desktop/merged.pdf')




SplitScore_usage = '''
============================================= SplitScore example commands =============================================

# SplitScore modules
TreeSAK SplitScore1     ->  Step 1: Infer gene tree
TreeSAK SplitScore1OMA  ->  Step 1: Infer gene tree (based on OMA outputs)
TreeSAK SplitScore2     ->  Step 2: Calculate split score

# SplitScore1
TreeSAK SplitScore1 -i OrthologousGroups.txt -s OrthologousGroupsFasta -o step1_op_dir -t 6 -f
TreeSAK SplitScore1 -i OrthologousGroups.txt -s OrthologousGroupsFasta -o step1_op_dir -t 6 -f -u interested_gnm.txt

# SplitScore2
# Please ensure that all the commands produced in step one have been executed before proceeding to step two.
TreeSAK SplitScore2 -i step1_op_dir -g gnm_cluster.tsv -k gnm_taxon.txt -f -t 10 -o step_2_op_dir

# As described in the Undinarchaeota paper (Nina Dombrowski 2020, NC)

=======================================================================================================================
'''



import argparse


sample_drep_gnms_usage = '''
============================= sample_drep_gnms example commands =============================

BioSAK sample_drep_gnms -c Cdb.csv -r rep_gnms.txt -k sponge_gnms.txt -o sampled_gnms.txt

=============================================================================================
'''


def cdb_2_gnm_cluster_file(Cdb_file):
    cluster_to_bin_dict = {}
    for each_bin in open(Cdb_file):
        if not each_bin.startswith('genome,secondary_cluster'):
            each_bin_split = each_bin.strip().split(',')
            bin_id = each_bin_split[0]
            secondary_cluster = each_bin_split[1]
            if secondary_cluster not in cluster_to_bin_dict:
                cluster_to_bin_dict[secondary_cluster] = [bin_id]
            else:
                cluster_to_bin_dict[secondary_cluster].append(bin_id)

    return cluster_to_bin_dict


def sample_drep_gnms(args):

    drep_cdb                     = args['c']
    drep_representative_gnm_txt  = args['r']
    gnm_to_keep_txt              = args['k']
    sampled_gnm_txt              = args['o']

    cluster_to_gnm_dict          = cdb_2_gnm_cluster_file(drep_cdb)
    drep_representative_gnm_list = [i.strip() for i in open(drep_representative_gnm_txt)]
    to_keep_gnm_set              = [i.strip() for i in open(gnm_to_keep_txt)]

    subsampled_gnm_set = set()
    for each_c in cluster_to_gnm_dict:
        cluster_gnms = cluster_to_gnm_dict[each_c]
        if len(cluster_gnms) == 1:
            subsampled_gnm_set.add(cluster_gnms[0])
        else:
            rep_g = ''
            g_in_to_keep_list = []
            for each_g in cluster_gnms:
                if each_g in drep_representative_gnm_list:
                    rep_g = each_g
                if each_g in to_keep_gnm_set:
                    g_in_to_keep_list.append(each_g)

            if len(g_in_to_keep_list) == 0:
                subsampled_gnm_set.add(rep_g)
            else:
                for each_to_keep_g in g_in_to_keep_list:
                    subsampled_gnm_set.add(each_to_keep_g)

    sampled_gnm_list_sorted = sorted([i for i in subsampled_gnm_set])
    sampled_gnm_list_sorted_no_ext = ['.'.join(i.split('.')[:-1]) for i in sampled_gnm_list_sorted]

    with open(sampled_gnm_txt, 'w') as sampled_gnm_txt_handle:
        sampled_gnm_txt_handle.write('\n'.join(sampled_gnm_list_sorted_no_ext))


if __name__ == '__main__':

    SliceMSA_parser = argparse.ArgumentParser()
    SliceMSA_parser.add_argument('-c',      required=True,  help='Cdb.csv')
    SliceMSA_parser.add_argument('-r',      required=True,  help='Id of drep representative genomes, with file extension')
    SliceMSA_parser.add_argument('-k',      required=True,  help='ID of genomes to keep, with file extension')
    SliceMSA_parser.add_argument('-o',      required=True,  help='ID of subsampled genomes')
    args = vars(SliceMSA_parser.parse_args())
    sample_drep_gnms(args)



import os
import glob
import argparse
from Bio import SeqIO
from Bio import AlignIO


ConcateMSA_usage = '''
================= ConcateMSA example commands =================

TreeSAK ConcateMSA -i aln -x aln -p concatenated -gene2gnm

# output file include:
concatenated.fasta
concatenated.phylip
concatenated.partition.txt

===============================================================
'''


def ConcateMSA(args):

    msa_dir                 = args['i']
    msa_ext                 = args['x']
    op_prefix               = args['p']
    gene2gnm                = args['gene2gnm']

    concatenated_msa_phy    = '%s.phylip'           % op_prefix
    concatenated_msa_fasta  = '%s.fasta'            % op_prefix
    partition_file          = '%s.partition.txt'    % op_prefix

    msa_file_re             = '%s/*.%s'             % (msa_dir, msa_ext)
    msa_file_list           = [os.path.basename(file_name) for file_name in glob.glob(msa_file_re)]
    msa_file_list_sorted    = sorted(msa_file_list)

    complete_gnm_set = set()
    for each_msa_file in msa_file_list:
        pwd_msa = '%s/%s' % (msa_dir, each_msa_file)
        for each_seq in SeqIO.parse(pwd_msa, 'fasta'):
            seq_id = each_seq.id
            if gene2gnm is True:
                seq_id = '_'.join(seq_id.split('_')[:-1])
            complete_gnm_set.add(seq_id)

    complete_gnm_list_sorted = sorted([i for i in complete_gnm_set])

    # initialize concatenated msa dict
    gnm_to_seq_dict = {i: '' for i in complete_gnm_list_sorted}
    msa_len_dict = dict()
    for each_msa_file in msa_file_list_sorted:
        msa_id = each_msa_file.split('.' + msa_ext)[0]

        # read in msa
        current_msa_len = 0
        current_msa_len_set = set()
        pwd_current_msa = '%s/%s' % (msa_dir, each_msa_file)
        current_msa_seq_dict = dict()
        for each_seq in SeqIO.parse(pwd_current_msa, 'fasta'):
            seq_id = each_seq.id
            if gene2gnm is True:
                seq_id = '_'.join(seq_id.split('_')[:-1])
            complete_gnm_set.add(seq_id)
            current_msa_seq_dict[seq_id] = str(each_seq.seq)
            current_msa_len_set.add(len(each_seq.seq))
            current_msa_len = len(each_seq.seq)

        if len(current_msa_len_set) != 1:
            print('Sequences with different length were found in %s, program exited!' % each_msa_file)
            exit()

        msa_len_dict[msa_id] = current_msa_len

        # add sequence to concatenated msa dict
        for each_gnm in complete_gnm_list_sorted:
            msa_seq = current_msa_seq_dict.get(each_gnm, current_msa_len*'-')
            gnm_to_seq_dict[each_gnm] += msa_seq

    # write out concatenated msa
    concatenated_msa_handle = open(concatenated_msa_fasta, 'w')
    for each_gnm in complete_gnm_list_sorted:
        concatenated_msa_handle.write('>%s\n' % each_gnm)
        concatenated_msa_handle.write('%s\n' % gnm_to_seq_dict[each_gnm])
    concatenated_msa_handle.close()

    # write out partition file
    end_pos = 0
    partition_file_handle = open(partition_file, 'w')
    for each_m in msa_file_list_sorted:
        gene_id = each_m.split('.' + msa_ext)[0]
        current_m_len = msa_len_dict[gene_id]
        partition_file_handle.write('%s = %s-%s\n' % (each_m, (end_pos + 1), (end_pos + current_m_len)))
        end_pos += current_m_len
    partition_file_handle.close()

    # convert msa in fasta to phy
    AlignIO.convert(concatenated_msa_fasta, 'fasta', concatenated_msa_phy, 'phylip-relaxed')



if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i',           required=True,                       help='input MSA folder')
    parser.add_argument('-x',           required=False, default='aln',       help='input file extension')
    parser.add_argument('-p',           required=True,                       help='output prefix')
    parser.add_argument('-gene2gnm',    required=False, action="store_true", help='gene id to gnm id, split sequence id before the last _')

    args = vars(parser.parse_args())
    ConcateMSA(args)



from sys import argv, stderr, version, exit
from re import sub

"""
MAD phylogenetic rooting
       - Please cite DOI:10.1038/s41559-017-0193

Usage:  
        mad filename [-mnsptfgvh]
    
    Where the file contains tree(s) in NEWICK format.
    Rooted tree(s) will be written to 'filename.rooted',
    rooting statistics to screen.

    Flags:
       -m: Multiple trees in input file, output file is verbose and
           contains '>' (info) and '<' (error) lines in addition to 
           NEWICK strings. 
           Default allows exactly one tree, and output is pure NEWICK.
       -n: Like -m, but pure newick. Only one rooted tree per input tree,
           errors as a lone ';' in a line.
       -s: Statistics - append MAD statistics to output NEWICK strings.
       -p: Polytomies - flat polytomies in rooted NEWICK (if present).         
           Default is a binary tree with polytomies arbitrarily resolved
           into zero-length branches.       
       -t: Tiny - retain branch lengths smaller than 10^-6.                
           Default contracts these to 0.0, thereby creating polytomies.  
  -f | -g: Figtree - rooted trees formatted for viewing AD scores in figtree.
           Important: after loading in figtree, please check the option 
           'Appearance>Gradient' manually, otherwise the branch colors will
           be misleading. -f reports ancestor deviations (AD) only for nodes,
           while -g reports also within-branch maximizing positions and AD values.           
       -v: Version.       
       -h: Help.
        
Please report bugs to giddy.landan@gmail.com .
"""

#v2.2 27-Mar-2018 
#=====================
#---- init

def mad(flags):

    fstr="".join([a.upper() for a in argv if a[0]=='-'])
    flags={f: fstr.count(f) for f in "MNSPTFGVHD"}

    if flags['V']:
        v=sub('\s+[^\[]+',' ',version,count=1)
        print('mad 2.2\npython '+v)
        exit()
    if flags['G'] and flags['F']:
        print('Flags -g and -f are mutually exclusive.')
        exit()
    if flags['M'] and flags['N']:
        print('Flags -m and -n are mutually exclusive.')
        exit()
    if len(argv)<2 or flags['H']:
        print(__doc__)
        exit()
    fstr=[a for a in argv if a[0]!='-']
    if len(fstr)!=2:
        exit('Expecting exactly one filename')
    print("\nMAD phylogenetic rooting")
    fn=fstr[1]
    #-------- consts
    if flags['T']: minlen=1e-15
    else:          minlen=1e-6
    madtol=1.0001
    gdbug_level=flags['D']
    at="#^$%"
    if flags['M'] or flags['N']:  err="Error: "
    else:           err="Error analyzing file '"+fn+"':\n\t"
    bug=err+"Oops... Please report this error to giddy.landan@gmail.com ."
    class madError(Exception):
        pass
    #------- short circut init - deffered imports
    try:
        with open(fn) as x: nwkstr=x.read()
    except FileNotFoundError:           exit("\nFile not found: "+fn)
    except:                             exit("\nError reading file: "+fn)
    blackspace=str.maketrans('', '', '\t\n\r\f\v')
    mnwk=nwkstr.translate(blackspace).split(';')
    if mnwk[-1]=='': mnwk.pop()
    nnwk=len(mnwk)
    if nnwk==0:                    exit(err+
                                        "No NEWICK string found.")
    if nnwk>1 and not flags['M'] and not flags['N']:  exit(err+
                                        "Expecting exactly one string terminated with ';'. (For multiple inputs, use -m.)")
    #---
    ofn=fn+".rooted"
    try:
        fh=open(ofn,'w')
    except:
        exit("\nError opening output file: "+ofn)
    #----------------
    #------ imports
    import re
    import numpy as np
    from copy import deepcopy
    from statistics import stdev, mean
    from inspect import stack
    #----- functions
    #========================================
    #===== write to output file
    def writeout(s):
        global fh,ofn
        try:
            fh.write(s+"\n")
        except:
            fh.close()
            exit("\nError writing to file: "+ofn)
    #<-- writeout
    #========================================

    #========================================
    #===== deal outputs
    def madlog(s):
        global fh,ofn,gdbug_level,flags
        if s[0]=='>':           pref=''
        elif s[:5]=='Error':    pref='<<< '
        else:                   pref='>> '
        if gdbug_level<1:
            pos=''
        else:
            sat=stack()[1][1:3]
            pos="{}:{} ".format(sat[0],sat[1])
        if pref=='' and flags['M']:
            writeout(s.strip())
        else:
            s1=re.sub('(?m)^',pref+pos,s.strip())
            if s1[0]=='<': s1+="\n"
            if flags['M']: writeout(s1)
            print(s1)
        if s[:5]=='Error':
            if flags['N']:
                writeout(';')
            raise madError
    #<-- madlog
    #========================================

    #========================================
    #===== debug messages
    def gdbug(lev,*args):
        global gdbug_level
        if lev>gdbug_level:
            return
        sat=stack()[1][1:3]
        print("---\n",file=stderr)
        [print(x,file=stderr) for x in args]
        print("<-- {}:{}".format(sat[0],sat[1]),file=stderr)
    #<-- gdbug
    #========================================

    #========================================
    #===== branch lengths conversion and sanity
    def str2len(s):
        try:
            x=float(s)
        except ValueError:      madlog(err+"Corrupt NEWICK format: invalid branch length (...'"+s+"'...).")
        if not np.isfinite(x):  madlog(err+"Cowardly refusing to root trees with infinite or undefined branch lengths (...'"+s+"'...).")
        return x
    #<-- str2len
    #========================================

    #========================================
    #======= convert NEWICK string, fills: labs,blen,nodes,trip + various scalars
    def nwk2tree(nwkstr):
        nc=[ nwkstr.count(x) for x in ';,:()']
        if nc[0]!=1 or nwkstr[-1]!=';':     madlog(err+
                                            "Corrupt NEWICK format - expecting exactly one string terminated with ';'.")
        if nc[3]!=nc[4]:                    madlog(err+
                                            "Corrupt NEWICK format - unbalanced ().")
        notu=nc[1]+1
        if notu<3:                          madlog(err+
                                            "Cowardly refusing to root trees with less than 3 OTUs.")
        if nc[2]>2*notu-2:                  madlog(err+
                                            "Corrupt NEWICK format - too many ':' | too few ','.")
        if nc[3]>notu-1:                    madlog(err+
                                            "Corrupt NEWICK format - too many '()' | too few ','.")
        nnode=notu*2-1
        rootnode=nnode-1
        nodes=np.full((nnode,3),-1,np.int16)
        trip=np.full((nnode,nnode),2,np.int8)
        blen=[0]*nnode
        labs=['']*nnode
        cld_splt=re.compile('\(([^()]*)\)')
        node_splt=re.compile('[:@]')
        iotu,tlen,ntiny,prec=[0]*4
        badbsp=[]
        inode=notu
        s2=nwkstr.replace("@",at).strip()
        while s2[0]=="(" :
            c3=cld_splt.split(s2,1)
            if len(c3)!=3:                  madlog(err+
                                            "Corrupt NEWICK format: () do not balance.")
            b2=c3[1].split(',')
            if len(b2)<2:                   madlog(err+
                                            "Corrupt NEWICK format: empty, singleton or unbalanced group ().")
            elif len(b2)>2:
                b2[1]="({},{}):0.0".format(b2[0],b2[1])
                z=str.join(',',b2[1:])
                s2="{}({}){}".format(c3[0],z,c3[2])
                continue
            for i in [0,1]:
                b=b2[i]
                if b[0]!="@":
                    z=b.split(":")
                    if len(z)!=2:           madlog(err+
                                            "Corrupt NEWICK format: malformed 'node_label:branch_length' clouse (...'"+b+"'...).")
                    if labs.count(z[0])>0:  madlog(err+
                                            "Cowardly refusing to root trees with duplicate OTU names ('"+z[0]+"').")
                    b="@{}@{}".format(iotu,b)
                    trip[iotu,iotu]=-1
                    iotu+=1
                ll=node_splt.split(b)
                if len(ll)!=4:              madlog(err+
                                            "Corrupt NEWICK format: malformed 'node_label:branch_length' clouse (...'"+ll[2].replace(at,"@")+"'...).")
                jnode=int(ll[1])
                labs[jnode]=ll[2]
                x=str2len(ll[3])
                if abs(x)<minlen and x!=0 and not flags['T']:
                    #gdbug(3,ll[3],x)
                    x=0
                    ntiny+=1
                if x<0:                     madlog(err+
                                            "Cowardly refusing to root trees with negative branch lengths.")
                blen[jnode]=x
                tlen+=x
                prec=max(prec,len(ll[3]))
                nodes[jnode,2]=inode
                nodes[inode,i]=jnode
                trip[inode,trip[jnode,:]<2]=i
            #<--i 0:1
            trip[inode,inode]=-1
            s2="{}@{}@{}".format(c3[0],inode,c3[2])
            inode+=1
        #<--inode loop
        n=sum([c3[2].count(x) for x in '(),:'])
        if n>0:                             madlog(err+
                                            "Corrupt NEWICK format: () balanced out before end of string, tail contains more '(),:'.\n"+
                                            "\t(Possible cause - missing ';' between two trees).")
        #gdbug(1,[notu,iotu,nnode,inode,tlen,prec,ntiny],badbsp)
        if inode!=nnode or iotu!=notu:
            #gdbug(2,s2,nwkstr,nodes)
            print(s2)
            exit()
            madlog(bug+' (Unidentified parsing error.)\nInput string: '+nwkstr)
        if  ntiny>0 and not flags['T']: madlog("Warning: {} tiny branch lengths (<10^-6) were converted to 0. (Override with '-t'.)".format(ntiny))
        n=sum([x>0 for x in blen])
        if n<3:                             madlog(err+
                                            "Cowardly refusing to root trees with less than 3 positive branch lengths.")
        if tlen==0:                         madlog(err+
                                            "Cowardly refusing to root zero-lengthed trees.")
        slen=np.sort(blen)
        n=sum(np.diff(slen[slen>0])==0)
        if n>0: madlog("Warning: Trees with repeating branch lengths are suspicious ({} repeating values).".format(n))
        if len(badbsp)>0: madlog("Warning: Non-numeric or negative bootstrap values ignored. ({}).".format(badbsp))
        #gdbug(2,nodes,blen,trip,labs)
        return(notu,nnode,rootnode,nodes,blen,trip,labs,prec,tlen)
    #<--nwk2tree()
    #========================================


    #========================================
    #-------- Pre-processing:
    def pwdist_preproc():
        #======= pairwise distances
        dist=np.full((nnode,nnode),0.0,np.float64)
        for i in range(notu,nnode):
            for j in [0,1]:
                b=nodes[i,j]
                for k in np.flatnonzero(trip[b,:]<2):
                    dist[i,k]=dist[b,k]+blen[b]
                    dist[k,i]=dist[i,k]
            for k0 in np.flatnonzero(trip[i,:]==0):
                for k1 in np.flatnonzero(trip[i,:]==1):
                    dist[k0,k1]=dist[i,k0]+dist[i,k1]
                    dist[k1,k0]=dist[k0,k1]
        #======= tip polytomies
        etrip=deepcopy(trip)
        n2n=list(range(nnode))
        ntipp=0
        for i in range(nnode):
            if n2n[i]<i:
                continue
            jj=np.flatnonzero(dist[i,:]==0)
            if len(jj)>1:
                for j in jj[1:]:
                    n2n[j]=i
                    if i<notu and j<notu:
                        etrip[j,:]=69
                        etrip[:,j]=69
                        ntipp+=1
        enotu=notu-ntipp
        npair=enotu*(enotu-1)/2
        #np.set_printoptions(threshold=np.inf)
        #gdbug(1,notu,ntipp,enotu,npair,n2n)
        if ntipp>0: madlog("Warning: Squeezing tip polytomies ({} OTUs, {} redundant tips, {} effective OTUs).".format(notu,ntipp,enotu))
        if enotu<3: madlog(err+"Cowardly refusing to root trees with less than 3 effctive OTUs.")
        #======= preproc in lists
        kij=[[]]*4
        dij=[[]]*nnode
        for i in range(nnode):
            for j in [0,1,2]:
                kij[j]=np.flatnonzero(etrip[i,:notu]==j)
            kij[3]=np.flatnonzero(etrip[i,:notu]<2)
            dij[i]=list([[]]*4)
            for j in [0,1,2,3]:
                dij[i][j]=dist[i,kij[j]].flatten().tolist()
        #gdbug(2,dij,"\n",npair)
        return(enotu,npair,dist,dij,n2n)
    #<-- pwdist_preproc()
    #========================================

    #========================================
    #-------- branch AD
    def ancestor_deviations():
        global dist
        #-------- node deviations triplets
        dsum=[[]]*nnode
        for i in range(nnode):
            dsum[i]=[0]*3
            if i<notu: continue
            for j in [0,1,2]:
                k1=(j+1)%3
                k2=(j+2)%3
                ndev=0
                for dik1 in dij[i][k1]:
                    for dik2 in dij[i][k2]:
                        d=dik1+dik2
                        if d>0:
                            ndev+=(2*dik1/d - 1)**2
                dsum[i][j]=ndev
        #<-- for i
        #-------- node ad
        nad=[-1]*nnode
        #nccv=[-1]*nnode
        #ndepth=[-1]*nnode
        #r2t=[]
        ibrn=range(notu,nnode)
        for i in range(nnode):
            if n2n[i]<i:
                nad[i]=nad[n2n[i]]
                continue
            if i<notu:
                nomin=enotu-1
            else:
                nomin=sum(dsum[i])
            for k in ibrn:
                if k!=i:
                    nomin+=dsum[k][trip[k,i]]
            #gdbug(2,nomin,npair)
            nad[i]=(nomin/npair)**0.5
            if nad[i]>1.0:
                madlog(bug+' (Node AD out of range.)\nInput tree is: '+nwkstr)
            #r2t=dij[i][3]+dij[i][2]
            #nccv[i]=stdev(r2t)/mean(r2t)
            #ndepth[i]=max(r2t)
        #-------- transversing pairs and branch ad
        ad=[10**6]*nnode
        rlen=[-1]*nnode
        rlen2=[-1]*nnode
        ccv=[-1]*nnode
        depth=[-1]*nnode
        polyroots=[-1]*nnode
        #gdbug(1,blen)
        for i in range(nnode-1):
            if blen[i]==0:
                rlen[i]=0
                rlen2[i]=0
                ad[i]=nad[i]+1
                continue
            j=nodes[i,2]
            jj=trip[j,i]
            denom=0
            nomin=0
            for dik1 in dij[i][3]:
                for dik2 in dij[i][2]:
                    d=dik1+dik2
                    d2=d**-2
                    denom+=d2
                    nomin+=d2*(dik2-dik1)
            rho=nomin/(2*denom)
            r1=min(max(0,rho),blen[i])
            r2=blen[i]-r1
            rn=-1
            if r1<minlen and r1<r2:
                r1=0
                r2=blen[i]
                rn=i
            elif r2<minlen and r2<r1:
                r1=blen[i]
                r2=0
                rn=j
            rlen[i]=r1
            rlen2[i]=r2
            if rn>-1 and polyroots[n2n[rn]]>-1:
                ad[i]=nad[rn]+1
                continue
            nomin=0
            for dik1 in dij[i][3]:
                for dik2 in dij[i][2]:
                    d=dik1+dik2
                    nomin+=(2*(dik1+r1)/d - 1)**2
            for k in ibrn:
                if trip[i,k]==2:
                    nomin+=dsum[k][trip[k,i]]
                else:
                    nomin+=dsum[k][trip[k,j]]
            ad[i]=(nomin/npair)**0.5
            if ad[i]>1.0:
                madlog(bug+' (AD out of range.)\nInput tree is: '+nwkstr)
            r2t=[ d+r1 for d in dij[i][3]]+[ d-r1 for d in dij[i][2]]
            ccv[i]=stdev(r2t)/mean(r2t)
            depth[i]=max(r2t)
            if rn>-1:
                polyroots[n2n[rn]]=i
        #gdbug(1,ad,nad,rlen,rlen2,ccv,polyroots)
        return(ad,nad,rlen,rlen2,ccv,depth)
    #<-- ancestor_deviations()
    #========================================

    #========================================
    #======= convert to NEWICK string
    def tree2nwk():
        global labs
        nstck=[rootnode,nodes[rootnode,0],nodes[rootnode,2]]
        nwk=[[]]*nnode
        nwk[nstck[1]]=[rootnode]
        nwk[nstck[2]]=[rootnode]
        #gdbug(2,"yy {} {} {} yy".format(rootnode,nwk,nstck))
        blens=[prec.format(x) for x in blen]
        if flags['F']:
            nads=["[&AD={:#5.3f},ADS={:#5.3f}]:".format(x,x) for x in nad]
        elif flags['G']:
            bads=["[&AD={:#5.3f},ADS={:#5.3f}]:".format(x,x) for x in ad]
            nads=["[&AD={:#5.3f},ADS={:#5.3f}]:".format(x,x) for x in nad]
            blens1=[prec.format(x) for x in rlen]
            blens2=[prec.format(x) for x in rlen2]
        else:
            nads=[":"]*nnode
        clen=0
        while len(nstck)>1:
            k=nstck[-1]
            #gdbug(3,k,nwk,nwk[k])
            if nstck.count(k)>1:
                #gdbug(9,nstck)
                madlog(bug)
            if k<notu:
                lens=blens[k]
                if flags['G']:
                    if rlen[k]==0:
                        nwk[k]="{}{}{}".format(labs[k],nads[k],blens2[k])
                    elif rlen2[k]==0:
                        nwk[k]="{}{}{}".format(labs[k],nads[k],blens1[k])
                    else:
                        nwk[k]="({}{}{}){}{}".format(labs[k],nads[k],blens1[k],bads[k],blens2[k])
                else:
                    nwk[k]="{}{}{}".format(labs[k],nads[k],blens[k])
                nstck.pop()
                #gdbug(3,'MM',k,nwk,blens,blen,nstck)
                clen+=blen[k]
                continue
            elif len(nwk[k])==1:
                for b in [0,1,2]:
                    if nodes[k,b]==nwk[k][0]:
                        continue
                    c=nodes[k,b]
                    nwk[c]=[k]
                    nstck.append(c)
                    nwk[k].append(c)
                continue
            else:
                b=nwk[k]
                f,c,d=b
                nwk[k]="{},{}".format(nwk[c],nwk[d])
                nwk[c],nwk[d]=["",""]
                if trip[k,f]==2:
                    k1=k
                else:
                    k1=f
                #gdbug(3,k,b[0],k1,blen[k1])
                if flags['G']:
                    if blen[k1]==0:
                        pass
                    else:
                        if rlen[k1]==0:
                            nwk[k]="({}){}{}".format(nwk[k],nads[k],blens2[k1])
                        elif rlen2[k1]==0:
                            nwk[k]="({}){}{}".format(nwk[k],nads[k],blens1[k1])
                        else:
                            if k1==k:
                                nwk[k]="(({}){}{}){}{}".format(nwk[k],nads[k],blens1[k1],bads[k1],blens2[k1])
                            else:
                                nwk[k]="(({}){}{}){}{}".format(nwk[k],nads[k],blens2[k1],bads[k1],blens1[k1])
                elif blen[k1]>0 or not flags['P']:
                    nwk[k]="({}){}{}".format(nwk[k],nads[k],blens[k1])
                #gdbug(3,"qq {}".format(blen[k1]))
                nstck.pop()
                clen+=blen[k1]
            #gdbug(1,'ww',k,nwk,blens,blen,nstck)
            #<--while stck
        tol=(tlen-clen)/tlen
        #gdbug(1,"tol=",tol)
        newnwk="({},{})".format(nwk[nodes[rootnode,0]],nwk[nodes[rootnode,2]])
        if abs(tol)>0.0000001:
                                        #gdbug(0,nodes,newnwk,nodes[rootnode,:],[rootnode,notu,newnwk.count(':')])
                                        madlog(bug+
                                        " tlen {} clen {} tol {}".format(tlen,clen,tol))
        #gdbug(2,newnwk,"\n----\n",nwk)
        return newnwk
    #<--tree2nwk()
    #========================================


    #========================================
    #====== fimd minimal values, generate AI, reroot
    def mad_output():
        global nodes,blen,trip,labs,ofn,rlen,rlen2,ad,nad
        #--------- find mads
        mad=min(ad)
        if mad<0.001: madlog("Warning: MAD=={:.5g} is too good to be true.".format(mad))
        roots = [i for i, x in enumerate(ad) if x <=(mad*madtol)]
        nroots=len(roots)
        if nroots==1:
            ai=mad/sorted(ad)[1]
        else:
            ai=1.0
        gdbug(1,mad,roots,ai)
        #----- output:
        #---- detach rootnode
        a,b=list(nodes[rootnode,0:2])
        #gdbug(1,nodes,a,b,blen,rlen,rlen2)
        nodes[a,2]=b
        nodes[b,2]=a
        blen[b]=blen[a]
        rlen[b]=rlen2[a]
        rlen2[b]=rlen[a]
        ad[b]=ad[a]
        ad[rootnode]=mad
        nad[rootnode]=mad
        if nroots>1:
            saved=[deepcopy(v) for v in [labs,blen,nodes,trip,ad,nad,rlen,rlen2]]
        rooted=""
        #gdbug(1,nodes,blen,rlen,rlen2)
        for r in range(nroots):
            if flags['N'] and nroots>1:
                r=np.argmin([ccv[j] for j in roots])
            i=roots[r]
            if i==rootnode:             madlog(bug+' (rootnode in roots.)')

            #reattach rootnode at inferred root
            j=nodes[i,2]
            k=trip[j,i]
            #gdbug(2,[i,j,k])
            #gdbug(3,nodes,blen,labs)
            nodes[i,2]=rootnode
            nodes[j,k]=rootnode
            nodes[rootnode,0]=i
            nodes[rootnode,1]=-1
            nodes[rootnode,2]=j
            trip[i,rootnode]=2
            trip[j,rootnode]=k
            trip[rootnode,i]=0
            trip[rootnode,j]=2
            if k==2:
                k1=j
                k2=rootnode
            else:
                k1=rootnode
                k2=j
            #blen[k1]=blen[i]-rlen[i]           #new branch length
            blen[k1]=rlen2[i]           #new branch length
            blen[i]=rlen[i]                    #new branch length
            rlen[rootnode]=0
            rlen2[rootnode]=rlen2[i]
            rlen[k1]=0
            rlen2[k1]=rlen2[i]
            rlen2[i]=0
            if blen[k1]==0 or blen[i]==0: madlog("Warning: Root is polytomous.")
            #gdbug(1,nodes,blen,labs)
                #------- make newick string
            rootstr=tree2nwk().replace(at,"@")
            s="[MAD={:#5.3f}_AI={:#5.3f}_CCV={:#.3g}%_N={}/{}]".format(mad,ai,ccv[i]*100,r+1,nroots)
            madlog(">> "+s)
            if flags['S']:
                rootstr+=s
            if flags['F'] or flags['G']:
                s='[&AD={:#5.3f},ADS="MAD={:#5.3f}",STT="CCV={:#.3g}%"]:{:.3g})[&ADS="AI={:#5.3f}"]'.format(mad,mad,ccv[i]*100,depth[i]*0.1,ai)
                rootstr='tree tree_{} = [&R] ({}{}'.format(r+1,rootstr,s);
            rooted+=rootstr+";"
            #gdbug(2,nwkstr,rooted)
            if flags['N']:
                break
            rooted+="\n"
            if r<nroots-1:
                labs,blen,nodes,trip,ad,nad,rlen,rlen2=[deepcopy(v) for v in saved]
        #<--r loop
        #--------
        rccv=", ".join(["{:#.3g}%".format(ccv[i]*100) for i in roots])
        sttstr="\nMinimal ancestor deviation, MAD = {:#5.3f}\n".format(mad)+ \
               "           Ambiguity index,  AI = {:#5.3f}\n".format(ai)+ \
               "                  Clock CV, CCV = {}".format(rccv)
        print(sttstr)
        s=""
        if flags['M']:
            if nroots>1: writeout(">> Tied root positions, {} rooted trees:".format(nroots))
            else:        writeout(">> Rooted tree:")
        if flags['F'] or flags['G']:
            rooted='#NEXUS\nbegin trees;\n'+rooted+"""end;
    begin figtree;
        set appearance.branchColorAttribute="AD";
        set appearance.branchLineWidth=6.0;
        set colour.scheme.ad="AD:HSBContinuous{{false,false,0,1},0.5,1.0,1.0,0.9,1.0,0.75,false}";
        set legend.attribute="AD";
        set legend.isShown=true;
        set nodeLabels.displayAttribute="ADS";
        set nodeLabels.isShown=true;
        set branchLabels.displayAttribute="STT";
        set branchLabels.isShown=true;
        set rectilinearLayout.alignTipLabels=true;
        set rectilinearLayout.curvature=9571;
        set rectilinearLayout.rootLength=0;
    """
            if flags['G']:
                rooted+="""
        set nodeShape.size=6.0;	
        set nodeShape.isShown=true;
    """
            rooted+='end;'
        writeout(rooted)
        #gdbug(1,rooted)
        if nroots>1:
            if flags['N']: print("Tied root positions,\n{} rooted trees, but just one written to {}\n".format(nroots,ofn))
            else:          print("Tied root positions,\n{} rooted trees written to {}\n".format(nroots,ofn))
        else:        print("Rooted tree written to '{}'\n".format(ofn));
    #<-- mad_output
    #--------------------------

    #--------------------------
    #----- Main loop
    for inwk in range(nnwk):
        if flags['M'] or flags['N']:
            print("\nAnalyzing tree {} of {} from '{}'...".format(inwk+1,nnwk,fn))
            if flags['M']: writeout(">Rooting of input tree #{}:".format(inwk+1))
        else: print("\nAnalyzing file '{}'...".format(fn))
        nwkstr=mnwk[inwk]+";"
        try:
            #---
            notu,nnode,rootnode,nodes,blen,trip,labs,prec,tlen = nwk2tree(nwkstr)
            #---
            gdbug(1,notu,nnode,rootnode,prec,tlen)
            gdbug(1,nodes,blen,trip,labs)
            prec="{"+":#.{}".format(min(prec-2,9))+"g}"
            #--- merge pre-existing root splits
            a,b=list(nodes[rootnode,0:2])
            blen[a]+=blen[b]
            blen[b]=0
            #gdbug(1,a,b,blen)
            #---
            enotu,npair,dist,dij,n2n=pwdist_preproc()
            #gdbug(1,enotu,npair)
            #---
            ad,nad,rlen,rlen2,ccv,depth=ancestor_deviations()
            #gdbug(2,ad)
            #---
            mad_output()
        except madError:
            pass
        except SystemExit:
            fh.close()
            raise
        except:
            madlog(bug+" (Unidentified error.)\n")
    #<-- inwk loop
    s="    - Please cite DOI:10.1038/s41559-017-0193\n"
    if flags['M']: writeout(">>"+s)
    print("\n"+s)
    fh.close()
    exit()
    #ffu if __name__ == '__main__':

    if __name__ == '__main__':
        mad_parser = argparse.ArgumentParser()
        mad_parser.add_argument('-i', required=False, help='input tree')
        mad_parser.add_argument('-m', required=False, help='input tree')
        mad_parser.add_argument('-n', required=False, help='input tree')
        mad_parser.add_argument('-s', required=False, help='input tree')
        mad_parser.add_argument('-p', required=False, help='input tree')
        mad_parser.add_argument('-t', required=False, help='input tree')
        mad_parser.add_argument('-f', required=False, help='input tree')
        mad_parser.add_argument('-g', required=False, help='input tree')
        mad_parser.add_argument('-v', required=False, help='input tree')
        args = vars(mad_parser.parse_args())
        mad(args)

'''

       -m: Multiple trees in input file, output file is verbose and contains '>' (info) and '<' (error) lines in addition to NEWICK strings. Default allows exactly one tree, and output is pure NEWICK.
       -n: Like -m, but pure newick. Only one rooted tree per input tree, errors as a lone ';' in a line.
       -s: Statistics - append MAD statistics to output NEWICK strings.
       -p: Polytomies - flat polytomies in rooted NEWICK (if present). Default is a binary tree with polytomies arbitrarily resolved into zero-length branches.       
       -t: Tiny - retain branch lengths smaller than 10^-6. Default contracts these to 0.0, thereby creating polytomies.  
  -f | -g: Figtree - rooted trees formatted for viewing AD scores in figtree. Important: after loading in figtree, please check the option 'Appearance>Gradient' manually, otherwise the branch colors will be misleading. -f reports ancestor deviations (AD) only for nodes, while -g reports also within-branch maximizing positions and AD values.           
       -v: Version.       

'''


import sys
import os
import argparse


def ale_parser(rec_folder, options):
    rec_files = [x for x in os.listdir(rec_folder) if x.endswith("uml_rec")]

    table_info = list()
    table_events = list()

    for rec_file in rec_files:
        with open(os.path.join(rec_folder, rec_file)) as f:
            fam = rec_file.replace(".ale.uml_rec", "")

            lines = f.readlines()
            stree = lines[2].strip()
            ll = lines[6].strip().split()[-1]
            dp, tp, lp = lines[8].strip().split("\t")[1:]
            n_reconciled_trees = int(lines[9].strip().split()[0])
            reconciled_trees = lines[11:n_reconciled_trees + 11]
            de, te, le, se = lines[11 + n_reconciled_trees + 1].split("\t")[1:]
            table = lines[11 + n_reconciled_trees + 3:]

        table_info.append((fam, ll, dp, tp, lp, de, te, le, se))
        table_events.append((fam, table))

    if options[0]:
        with open("SpeciesTreeRef.newick", "w") as f:
            f.write(stree.split("\t")[-1])

    if options[1]:

        with open("TableInfo.tsv", "w") as f:
            head = "\t".join(["Family", "LL", "Dp", "Tp", "Lp", "De", "Te", "Le", "Se"]) + "\n"
            f.write(head)
            for info in table_info:
                f.write("\t".join(info))

    if options[2]:

        with open("TableEvents.tsv", "w") as f:

            header = "Family\tBranchType\t" + table[0].replace("# of", "Branch")
            f.write(header)

            for fam, events in table_events:
                for b in events[1:]:
                    f.write(fam + "\t" + b)

    if options[3]:
        with open("GeneTrees.nwk", "w") as f:
            for t in reconciled_trees:
                f.write(t)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("-i", help="Folder with uml_rec files")
    parser.add_argument("-s", help="Prints species tree to a different file", action='store_true', default=False)
    parser.add_argument("-f", help="Prints info about the family (LogLikelihood, probabilities and total number of events, to a different file", action='store_true', default=False)
    parser.add_argument("-t", help="Prints reconciliation table to a different file", action='store_true', default=False)
    parser.add_argument("-g", help="Prints gene trees to a different file", action='store_true', default=False)

    args = parser.parse_args()

    if args.i == None:
        print("use python ale_parser.py -h to see options")
        print("you can run this script just with python ale_parser -i FolderWithRecFiles -sft")
        exit(0)

    ale_parser(args.i, [args.s, args.f, args.t, args.g])




import os

# extract path to the config file
pwd_config_file = os.path.realpath(__file__)
config_file_path = '/'.join(pwd_config_file.split('/')[:-1])

# specify full path to corresponding executables at the right side of colon
config_dict = {'config_file_path'       : config_file_path,
               'prodigal'               : 'prodigal',
               'hmmsearch'              : 'hmmsearch',
               'hmmfetch'               : 'hmmfetch',
               'hmmalign'               : 'hmmalign',
               'hmmstat'                : 'hmmstat',
               'mafft'                  : 'mafft',
               'bowtie2'                : 'bowtie2',
               'bowtie2_build'          : 'bowtie2-build',
               'blastp'                 : 'blastp',
               'blastn'                 : 'blastn',
               'makeblastdb'            : 'makeblastdb',
               'fasttree'               : 'FastTree',
               'ranger_mac'             : '%s/Ranger-DTL-Dated.mac'           % config_file_path,
               'ranger_linux'           : '%s/Ranger-DTL-Dated.linux'         % config_file_path,
               'path_to_hmm'            : '%s/MetaCHIP_phylo.hmm'             % config_file_path,
               'circos_HGT_R'           : '%s/MetaCHIP_circos_HGT.R'          % config_file_path,
               'VisHPD95_R'             : '%s/VisHPD95.R'                     % config_file_path,
               'label_tree_R'           : '%s/label_tree.R'                   % config_file_path,
               'cdd2cog_perl'           : '%s/cdd2cog.pl'                     % config_file_path,
               'get_sankey_plot_R'      : '%s/get_sankey_plot.R'              % config_file_path,
               'compare_trees_R'        : '%s/compare_trees.R'                % config_file_path,
               'ko00001_keg'            : '%s/ko00001.keg'                    % config_file_path,
               'MetaCyc_rxns_with_ec'   : '%s/MetaCyc_reactions_with_ec.txt'  % config_file_path
               }



import os
import argparse
from Bio import SeqIO
from distutils.spawn import find_executable


GeneTree_usage = '''
============= GeneTree example commands =============

TreeSAK GeneTree -i amoA.faa -o amoA_tree -t 36 -f

=====================================================
'''


def check_dependencies(program_list):

    not_detected_programs = []
    for needed_program in program_list:
        if find_executable(needed_program) is None:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not found, program exited!' % ','.join(not_detected_programs))
        exit()


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def filter_by_gap(file_in, max_gap_pct, file_out):
    file_out_handle = open(file_out, 'w')
    for each_seq in SeqIO.parse(file_in, 'fasta'):
        seq_str = str(each_seq.seq)
        gap_num = seq_str.count('-')
        gap_pct = gap_num*100 / len(seq_str)
        if gap_pct <= float(max_gap_pct):
            file_out_handle.write('>%s\n%s\n' % (each_seq.id, seq_str))
    file_out_handle.close()


def GeneTree(args):

    seq_file                    = args['i']
    num_threads                 = args['t']
    op_dir                      = args['o']
    force_create_op_dir         = args['f']
    gap_cutoff                  = args['max_gap']
    trim_with_trimal            = args['trimal']
    bmge_trim_model             = 'BLOSUM30'
    bmge_entropy_score_cutoff   = '0.55'

    # check dependencies
    if trim_with_trimal is False:
        check_dependencies(['mafft-einsi', 'java'])
    else:
        check_dependencies(['mafft-einsi', 'trimal'])

    # specify path to BMGE.jar
    current_file_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar      = '%s/BMGE.jar' % current_file_path

    # determine the version of iqtree available on the system
    if find_executable('iqtree2'):
        iqtree_exe = 'iqtree2'
    elif find_executable('iqtree'):
        iqtree_exe = 'iqtree'
    else:
        print('iqtree not detected, program exited!')
        exit()

    # create op_dir
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    ######################################## define output file name ########################################

    sep_file_path, sep_file_base, sep_file_ext = sep_path_basename_ext(seq_file)
    get_gene_tree_cmds_txt          = '%s/cmds.txt'                 % op_dir
    msa_file                        = '%s/%s.aln'                   % (op_dir, sep_file_base)

    msa_file_trimmed                = '%s/%s.bmge.aln'              % (op_dir, sep_file_base)
    msa_file_trimmed_low_gap        = '%s/%s.bmge.maxgap%s.aln'     % (op_dir, sep_file_base, gap_cutoff)
    if trim_with_trimal is True:
        msa_file_trimmed            = '%s/%s.trimal.aln'            % (op_dir, sep_file_base)
        msa_file_trimmed_low_gap    = '%s/%s.trimal.maxgap%s.aln'   % (op_dir, sep_file_base, gap_cutoff)

    #########################################################################################################

    # prepare commands
    mafft_cmd       = 'mafft-einsi --thread %s --quiet %s > %s'              % (num_threads, seq_file, msa_file)
    trim_cmd        = 'java -jar %s -i %s -m %s -t AA -h %s -of %s'          % (pwd_bmge_jar, msa_file, bmge_trim_model, bmge_entropy_score_cutoff, msa_file_trimmed)
    if trim_with_trimal is True:
        trim_cmd    = 'trimal -in %s -out %s -automated1'                    % (msa_file, msa_file_trimmed)
    iqtree_cmd      = '%s -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s/%s' % (iqtree_exe, num_threads, msa_file_trimmed_low_gap, op_dir, sep_file_base)

    # write out commands
    with open(get_gene_tree_cmds_txt, 'w') as f:
        f.write('%s\n%s\n%s\n' % (mafft_cmd, trim_cmd, iqtree_cmd))

    # run mafft
    print(mafft_cmd)
    os.system(mafft_cmd)

    # run BMGE
    print(trim_cmd)
    os.system(trim_cmd)

    # remove high gap sequences
    filter_by_gap(msa_file_trimmed, gap_cutoff, msa_file_trimmed_low_gap)

    # run iqtree
    print(iqtree_cmd)
    os.system(iqtree_cmd)

    print('Done!')


if __name__ == '__main__':

    GeneTree_parser = argparse.ArgumentParser()
    GeneTree_parser.add_argument('-i',          required=False, default=None,           help='sequence file')
    GeneTree_parser.add_argument('-o',          required=True,                          help='output dir')
    GeneTree_parser.add_argument('-t',          required=False, type=int, default=1,    help='number of threads, default is 1')
    GeneTree_parser.add_argument('-trimal',     required=False, action="store_true",    help='trim with trimal, default is BMGE')
    GeneTree_parser.add_argument('-f',          required=False, action="store_true",    help='force overwrite')
    GeneTree_parser.add_argument('-max_gap',    required=False, default='40',           help='maximum percentage of gap, default is 40')
    args = vars(GeneTree_parser.parse_args())
    GeneTree(args)



import os
import glob
import math
import random
import argparse
import seaborn as sns
from Bio import SeqIO
from ete3 import Tree
from PIL import Image
from itolapi import Itol
import multiprocessing as mp
from PyPDF3.pdf import PageObject
from PyPDF3 import PdfFileWriter, PdfFileReader
from ete3 import TextFace, TreeStyle, NodeStyle


ALE_usage = '''
========================= ALE example commands =========================

TreeSAK ALE1  ->  step 1: get gene tree
TreeSAK ALE2  ->  step 2: run ALE
TreeSAK ALE3  ->  step 3: parse ALE output
TreeSAK ALE4  ->  Infer ancestral genome

cd /Users/songweizhi/Desktop/demo
TreeSAK ALE1 -i OrthologousGroups.txt -s combined_d__Archaea_o_rs.faa -p oma -c genome_taxon.txt -m 50 -n 2 -t 6 -jt 3 -f -o ALE1_op_dir
TreeSAK ALE2 -i ALE1_op_dir -s Marker_set_1_PA_75_C60_PMSF_concatenated_rooted.treefile -c genome_taxon.txt -t 6 -f -o ALE2_op_dir
TreeSAK ALE3 -i1 ALE1_op_dir -i2 ALE2_op_dir -c genome_taxon.txt -color ar_phylum_color_code.txt -o ALE3_op_dir_0.3 -fc 0.3 -f -api S1kZZuDHc0d5M7J5vLnUNQ
TreeSAK ALE3 -i1 ALE1_op_dir -i2 ALE2_op_dir -c genome_taxon.txt -color ar_phylum_color_code.txt -o ALE3_op_dir_0.5 -fc 0.5 -f -api S1kZZuDHc0d5M7J5vLnUNQ
TreeSAK ALE3 -i1 ALE1_op_dir -i2 ALE2_op_dir -c genome_taxon.txt -color ar_phylum_color_code.txt -o ALE3_op_dir_0.8 -fc 0.8 -f -api S1kZZuDHc0d5M7J5vLnUNQ


ALEobserve OMA00003_for_ALE.ufboot


cd /Users/songweizhi/Desktop/demo/subset_gnm_tree_no
docker run -v $PWD:$PWD -w $PWD gregmich/alesuite_new ALEobserve OMA00003_for_ALE.ufboot
docker run -v $PWD:$PWD -w $PWD gregmich/alesuite_new ALEml_undated genome_tree_rooted.treefile OMA00003_for_ALE.ufboot.ale

cd /Users/songweizhi/Desktop/demo/subset_gnm_tree_yes
docker run -v $PWD:$PWD -w $PWD gregmich/alesuite_new ALEobserve OMA00003_for_ALE.ufboot
docker run -v $PWD:$PWD -w $PWD gregmich/alesuite_new ALEml_undated OMA00003_genome_tree_for_ALE.treefile OMA00003_for_ALE.ufboot.ale



========================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def select_seq(arg_list):

    seq_file    = arg_list[0]
    id_file     = arg_list[1]
    output_file = arg_list[2]

    seq_id_set = {i.strip() for i in open(id_file)}
    output_file_handle = open(output_file, 'w')
    for seq_record in SeqIO.parse(seq_file, 'fasta'):
        if seq_record.id in seq_id_set:
            SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
    output_file_handle.close()


def root_with_out_group(tree_file, out_group_txt, tree_file_rooted):

    out_group_set = set()
    for each_og in open(out_group_txt):
        out_group_set.add(each_og.strip())

    tre = Tree(tree_file, format=1)
    out_group_lca = tre.get_common_ancestor(out_group_set)
    tre.set_outgroup(out_group_lca)
    tre.write(outfile=tree_file_rooted)


def subset_tree(tree_file_in, leaves_to_keep_list, tree_file_out):

    input_tree = Tree(tree_file_in)
    subset_tree = input_tree.copy()
    subset_tree.prune(leaves_to_keep_list, preserve_branch_length=True)
    if tree_file_out is None:
        return subset_tree.write()
    else:
        subset_tree.write(outfile=tree_file_out)


def plot_tree(input_tree, tree_title, node_label_dict, node_label_color_dict, align_leaf_label, show_scale, output_plot):

    if os.path.isfile(input_tree) is False:
        print('Tree file not found, program exited!')
        print(input_tree)
        exit()

    t = Tree(input_tree)
    ts = TreeStyle()
    ts.mode = "r"                       # tree model: 'r' for rectangular, 'c' for circular
    ts.show_border = False              # set tree image border
    ts.show_leaf_name = False           # show/hide leaf name, hide here, so you can customise it below with node.add_face()
    ts.title.add_face(TextFace(tree_title, fsize=9, fgcolor='black', ftype='Arial', tight_text=False), column=0)  # add tree title

    # set node style
    for each_node in t.traverse():
        ns = NodeStyle()
        ns["shape"]         = "circle"  # dot shape: circle, square or sphere
        ns["fgcolor"]       = "black"   # color of shape(not label)
        ns['size']          = 0         # node shape size
        ns['hz_line_type']  = 0         # horizontal branch line type: 0 for solid, 1 for dashed, 2 for dotted
        ns['vt_line_type']  = 0         # vertical branch line type:   0 for solid, 1 for dashed, 2 for dotted
        ns['hz_line_width'] = 0.5       # horizontal branch line width
        ns['vt_line_width'] = 0.5       # vertical branch line width

        leaf_label_position = 'branch-right'
        if align_leaf_label is True:
            leaf_label_position = 'aligned'

        if each_node.is_leaf():
            node_id = each_node.name
            node_label_color = node_label_color_dict.get(node_id, 'black')
            node_label_text  = node_label_dict.get(node_id, node_id)
            each_node.add_face(TextFace(node_label_text, fsize=8, fgcolor=node_label_color, tight_text=False, bold=False),
                               column=0, position=leaf_label_position)  # aligned, branch-right
        else:
            pass
        each_node.set_style(ns)

    # set layout
    ts.rotation                 = 0             # from 0 to 360
    ts.margin_top               = 10            # top tree image margin
    ts.margin_bottom            = 10            # bottom tree image margin
    ts.margin_left              = 10            # left tree image margin
    ts.margin_right             = 10            # right tree image margin
    ts.branch_vertical_margin   = 3             # 3 pixels between adjancent branches
    ts.show_scale               = show_scale    # show_scale
    ts.show_border              = False         # set tree image border

    # write out tree
    t.render(output_plot, w=1200, units="px", tree_style=ts)


def merge_image(image_file_list, output_image):

    images = [Image.open(x) for x in image_file_list]
    widths, heights = zip(*(i.size for i in images))
    total_width = sum(widths)
    max_height = max(heights)
    new_im = Image.new('RGB', (total_width, max_height), color='white')

    x_offset = 0
    for im in images:
        new_im.paste(im, (x_offset, 0))
        x_offset += im.size[0]

    new_im.save(output_image)


def merge_pdf(pdf_1, pdf_2, margin_size, op_pdf):

    page1 = PdfFileReader(open(pdf_1, "rb"), strict=False).getPage(0)
    page2 = PdfFileReader(open(pdf_2, "rb"), strict=False).getPage(0)

    total_width  = page1.mediaBox.upperRight[0] + page2.mediaBox.upperRight[0] + margin_size*3
    total_height = max([page1.mediaBox.upperRight[1], page2.mediaBox.upperRight[1]]) + margin_size*2

    new_page = PageObject.createBlankPage(None, total_width, total_height)
    new_page.mergeTranslatedPage(page1, margin_size, (total_height-margin_size-page1.mediaBox.upperRight[1]))
    new_page.mergeTranslatedPage(page2, (page1.mediaBox.upperRight[0] + margin_size*2), margin_size)

    output = PdfFileWriter()
    output.addPage(new_page)
    output.write(open(op_pdf, "wb"))


def uts_to_itol_connections(genome_tree_file, ale_formatted_gnm_tree, interal_node_prefix, uts_file, freq_cutoff, ignore_leaf_hgt, ignore_vertical_hgt, donor_node_min_leaf_num, recipient_node_min_leaf_num, itol_connection_txt, dr_separator):

    # get internal_node_to_leaf_dict
    internal_node_to_leaf_dict = get_node_to_leaf_dict(ale_formatted_gnm_tree)

    paired_donor_to_recipient_leaf_dict = dict()
    qualified_hgt_num = 0

    leaf_id_set = []
    if os.path.isfile(genome_tree_file):
        leaf_id_set = [i.name for i in Tree(genome_tree_file, format=3).get_leaves()]
    else:
        print('%s not found!' % genome_tree_file)

    hgt_freq_dict = dict()
    connection_line_to_write_dict = dict()
    with open(itol_connection_txt, 'w') as itol_connection_txt_handle:
        itol_connection_txt_handle.write('DATASET_CONNECTION\nSEPARATOR TAB\nDATASET_LABEL\tdemo_connections\n')
        itol_connection_txt_handle.write('COLOR\t#ff0ff0\nDRAW_ARROWS\t1\nARROW_SIZE\t60\nLOOP_SIZE\t100\n')
        itol_connection_txt_handle.write('MAXIMUM_LINE_WIDTH\t10\nCURVE_ANGLE\t45\nCENTER_CURVES\t1\nALIGN_TO_LABELS\t0\nDATA\n')
        for each_line in open(uts_file):
            if not each_line.startswith('#'):
                each_line_split = each_line.strip().split('\t')
                donor = each_line_split[0]
                recipient = each_line_split[1]
                freq = float(each_line_split[2])

                # add prefix to internal donor node
                if donor in leaf_id_set:
                    donor_with_prefix = donor
                else:
                    donor_with_prefix = interal_node_prefix + donor

                # add prefix to internal recipient node
                if recipient in leaf_id_set:
                    recipient_with_prefix = recipient
                else:
                    recipient_with_prefix = interal_node_prefix + recipient

                key_str      = '%s%s%s' % (donor_with_prefix, dr_separator, recipient_with_prefix)

                line_to_write = ''
                if freq >= freq_cutoff:
                    if ignore_leaf_hgt is False:
                        if ignore_vertical_hgt is False:
                            line_to_write = '%s\t%s\t%s\t%s\t%s\t%s->%s(%s)\n' % (donor_with_prefix, recipient_with_prefix, freq, '#EB984E', 'normal', donor_with_prefix, recipient_with_prefix, freq)
                            qualified_hgt_num += 1
                        else:
                            donor_is_ancestor_of_recipient = check_a_is_ancestor_of_b(ale_formatted_gnm_tree, donor, recipient)
                            donor_is_child_of_recipient    = check_a_is_child_of_b(ale_formatted_gnm_tree, donor, recipient)
                            if (donor_is_ancestor_of_recipient is False) and (donor_is_child_of_recipient is False):
                                line_to_write = '%s\t%s\t%s\t%s\t%s\t%s->%s(%s)\n' % (donor_with_prefix, recipient_with_prefix, freq, '#EB984E', 'normal', donor_with_prefix, recipient_with_prefix, freq)
                                qualified_hgt_num += 1
                    else:
                        if (each_line_split[0] not in leaf_id_set) and (each_line_split[1] not in leaf_id_set):
                            donor_node_leaf_num = len(internal_node_to_leaf_dict.get(donor, []))
                            recipient_node_leaf_num = len(internal_node_to_leaf_dict.get(recipient, []))
                            if (donor_node_leaf_num >= donor_node_min_leaf_num) and (recipient_node_leaf_num >= recipient_node_min_leaf_num):
                                if ignore_vertical_hgt is False:
                                    line_to_write = '%s\t%s\t%s\t%s\t%s\t%s->%s(%s)\n' % (donor_with_prefix, recipient_with_prefix, freq, '#EB984E', 'normal', donor_with_prefix, recipient_with_prefix, freq)
                                    qualified_hgt_num += 1
                                else:
                                    donor_is_ancestor_of_recipient = check_a_is_ancestor_of_b(ale_formatted_gnm_tree, donor, recipient)
                                    donor_is_child_of_recipient    = check_a_is_child_of_b(ale_formatted_gnm_tree, donor, recipient)
                                    if (donor_is_ancestor_of_recipient is False) and (donor_is_child_of_recipient is False):
                                        line_to_write = '%s\t%s\t%s\t%s\t%s\t%s->%s(%s)\n' % (donor_with_prefix, recipient_with_prefix, freq, '#EB984E', 'normal', donor_with_prefix, recipient_with_prefix, freq)
                                        qualified_hgt_num += 1
                                        paired_donor_to_recipient_leaf_dict[key_str] = [internal_node_to_leaf_dict.get(donor, []), internal_node_to_leaf_dict.get(recipient, [])]

                if line_to_write != '':
                    itol_connection_txt_handle.write(line_to_write)
                    connection_line_to_write_dict[key_str] = line_to_write
                    hgt_freq_dict[key_str] = freq

    combined_connection_file_path, combined_connection_file_basename, combined_connection_file_ext = sep_path_basename_ext(itol_connection_txt)

    # write out connections separately
    for each_connection in connection_line_to_write_dict:
        pwd_connection_txt = '%s/%s_%s.txt' % (combined_connection_file_path, combined_connection_file_basename, each_connection)
        pwd_connection_txt_handle = open(pwd_connection_txt, 'w')
        pwd_connection_txt_handle.write('DATASET_CONNECTION\nSEPARATOR TAB\nDATASET_LABEL\tdemo_connections\n')
        pwd_connection_txt_handle.write('COLOR\t#ff0ff0\nDRAW_ARROWS\t1\nARROW_SIZE\t60\nLOOP_SIZE\t100\n')
        pwd_connection_txt_handle.write('MAXIMUM_LINE_WIDTH\t10\nCURVE_ANGLE\t45\nCENTER_CURVES\t1\nALIGN_TO_LABELS\t0\nDATA\n')
        pwd_connection_txt_handle.write(connection_line_to_write_dict[each_connection] + '\n')
        pwd_connection_txt_handle.close()

    return internal_node_to_leaf_dict, paired_donor_to_recipient_leaf_dict, hgt_freq_dict


def itol_tree(tree_file, annotation_file_list, project_name, APIkey, display_mode, op_plot):

    # https://github.com/albertyw/itolapi
    # http://itol.embl.de/help.cgi#batch

    op_plot_ext = op_plot.split('.')[-1]

    # upload tree to iTOL
    itol_uploader = Itol()
    itol_uploader.params['projectName'] = project_name  # better to create a project with a unique name.
    itol_uploader.params['APIkey'] = APIkey  # sine we are the same account, we can use the same APIkey
    itol_uploader.params['treeName'] = tree_file
    itol_uploader.add_file(tree_file)

    # upload annotation files to iTOL
    for annotation_file in annotation_file_list:
        itol_uploader.add_file(annotation_file)

    status = itol_uploader.upload()
    # import pdb;pdb.set_trace()
    assert status != False

    # the following parameters are optional, refer to https://itol.embl.de/help.cgi#batchExp
    if len(annotation_file_list) == 1:
        datasets_visible_str = '0'
    elif len(annotation_file_list) == 2:
        datasets_visible_str = '0,1'
    elif len(annotation_file_list) == 3:
        datasets_visible_str = '0,1,2'
    else:
        datasets_visible_str = ','.join([str(i) for i in list(range(0, len(annotation_file_list)))])
    itol_exporter = itol_uploader.get_itol_export()
    itol_exporter.set_export_param_value('datasets_visible', datasets_visible_str)
    itol_exporter.set_export_param_value('display_mode', display_mode)
    itol_exporter.set_export_param_value('range_mode', '2')
    itol_exporter.set_export_param_value('dashed_lines', '1')
    # itol_exporter.set_export_param_value('current_font_size', '96')
    itol_exporter.set_export_param_value('line_width', '3')
    itol_exporter.set_export_param_value('vertical_shift_factor', '0.9')
    itol_exporter.set_export_param_value('horizontal_scale_factor', '0.9')
    itol_exporter.set_export_param_value('format', op_plot_ext)
    itol_exporter.export(op_plot)


def get_node_to_leaf_dict(tree_file):
    internal_node_to_leaf_dict = dict()
    for node in Tree(tree_file, format=1).traverse():
        if not node.is_leaf():
            node_name = node.name
            node_leaf_list = node.get_leaf_names()
            internal_node_to_leaf_dict[node_name] = node_leaf_list
    return internal_node_to_leaf_dict


def combine_trees(t1_with_len, t2_with_name, op_tree_with_both):

    # assume t1 has brancn length
    # assume t2 has internal node name

    t1 = Tree(t1_with_len, format=0)
    t2 = Tree(t2_with_name, format=1)

    t1_leaves_to_node_dict = dict()
    for t1_node in t1.traverse():
        leaf_str = '__'.join(sorted(list(t1_node.get_leaf_names())))
        t1_leaves_to_node_dict[leaf_str] = t1_node

    t2_leaves_to_node_dict = dict()
    for t2_node in t2.traverse():
        leaf_str = '__'.join(sorted(list(t2_node.get_leaf_names())))
        t2_leaves_to_node_dict[leaf_str] = t2_node

    t1_node_to_t2_node_dict = dict()
    for index, t1_node in t1_leaves_to_node_dict.items():
        t2_node = t2_leaves_to_node_dict[index]
        t1_node_to_t2_node_dict[t1_node] = t2_node

    merged_tree = t1.copy()
    for node, t1_node in zip(merged_tree.traverse(), t1.traverse()):
        node.name = t1_node_to_t2_node_dict[t1_node].name
    merged_tree.write(outfile=op_tree_with_both, format=3)


def prefix_internal_nodes(tree_in, prefix_str, tree_out):
    t = Tree(tree_in, format=3)
    t_renamed = t.copy()
    for node in t_renamed.traverse():
        if not node.is_leaf():
            node_name_prefixed = '%s%s' % (prefix_str, node.name)
            node.name = node_name_prefixed
        t_renamed.write(outfile=tree_out, format=3)


def check_a_is_ancestor_of_b(tree_file, node_a, node_b):

    a_is_ancestor_of_b = False
    for node in Tree(tree_file, format=1).traverse():
        node_name = node.name
        if node_name == node_b:
            node_ancestor_list = [i.name for i in node.get_ancestors()]
            if node_a in node_ancestor_list:
                a_is_ancestor_of_b = True

    return a_is_ancestor_of_b


def check_a_is_child_of_b(tree_file, node_a, node_b):

    a_is_child_of_b = False
    for node in Tree(tree_file, format=1).traverse():
        node_name = node.name
        if node_name == node_b:
            node_children_list = [i.name for i in node.get_descendants()]
            if node_a in node_children_list:
                a_is_child_of_b = True

    return a_is_child_of_b


def root_at_midpoint(tree_in, tree_in_rooted):
    t = Tree(tree_in)
    midpoint = t.get_midpoint_outgroup()
    t.set_outgroup(midpoint)
    t.write(outfile=tree_in_rooted)


def get_color_list(color_num):
    if color_num <= 8:
        color_list_combined = ['#3787c0', '#39399f', '#ffb939', '#399f39', '#9f399f', '#fb694a', '#9f9f39', '#959595']

    elif 8 < color_num <= 16:
        color_list_combined = ['#2b7bba', '#89bedc', '#2e2e99', '#8a8acc', '#ffa500', '#ffc55c', '#2e992e', '#8acc8a',
                               '#992e99', '#cc8acc', '#d52221', '#fc8161', '#99992e', '#cccc8a', '#5c5c5c', '#adadad']
    else:
        color_num_each = math.ceil(color_num / 8) + 2
        color_list_1 = sns.color_palette('Blues', n_colors=color_num_each).as_hex()
        color_list_2 = sns.light_palette('navy', n_colors=color_num_each).as_hex()
        color_list_3 = sns.light_palette('orange', n_colors=color_num_each).as_hex()
        color_list_4 = sns.light_palette('green', n_colors=color_num_each).as_hex()
        color_list_5 = sns.light_palette('purple', n_colors=color_num_each).as_hex()
        color_list_6 = sns.color_palette('Reds', n_colors=color_num_each).as_hex()
        color_list_7 = sns.light_palette('olive', n_colors=color_num_each).as_hex()
        color_list_8 = sns.color_palette('Greys', n_colors=color_num_each).as_hex()

        color_list_combined = []
        for color_list in [color_list_1, color_list_2, color_list_3, color_list_4, color_list_5, color_list_6,
                           color_list_7, color_list_8]:
            for color in color_list[2:][::-1]:
                color_list_combined.append(color)

    color_list_to_return = random.sample(color_list_combined, color_num)

    color_list_to_return_sorted = []
    for color_to_return in color_list_combined:
        if color_to_return in color_list_to_return:
            color_list_to_return_sorted.append(color_to_return)

    return color_list_to_return_sorted


def scale_str_to_size_list(scale_str):
    scale_list = scale_str.split('-')
    scale_list = [float(i) for i in scale_list]

    shape_size_list = []
    if scale_list[0] == 0:
        shape_size_list = [0]
        for each_value in scale_list[1:-1]:
            current_size = each_value / scale_list[-1]
            shape_size_list.append(current_size)
        shape_size_list.append(1)

    if scale_list[0] != 0:
        shape_size_list = [0.1]
        interval_num = len(scale_list) - 1
        interval_value = (1 - 0.1) / interval_num
        n = 1
        for each_value in scale_list[1:-1]:
            shape_size_list.append(interval_value * n + 0.1)
            n += 1
        shape_size_list.append(1)

    return shape_size_list


def get_ortho_to_gene_dict(ortho_groups_txt, og_program):

    ortho_to_gene_dict = dict()
    for each_og in open(ortho_groups_txt):
        if not each_og.startswith('#'):
            og_id = ''
            gene_list = []
            if og_program == 'orthofinder':
                each_og_split = each_og.strip().split(' ')
                og_id = each_og_split[0][:-1]
                gene_list = each_og_split[1:]
            elif og_program == 'oma':
                each_og_split = each_og.strip().split('\t')
                og_id = each_og_split[0]
                group_member_list = each_og_split[1:]
                for each_protein in group_member_list:
                    protein_id = each_protein.split(' ')[0].split(':')[1]
                    gene_list.append(protein_id)
            ortho_to_gene_dict[og_id] = gene_list

    return ortho_to_gene_dict


def prepare_ale_ip_worker(arg_list):

    qualified_og                    = arg_list[0]
    gene_tree_dir                   = arg_list[1]
    ale_wd                          = arg_list[2]
    genome_tree_file_rooted         = arg_list[3]
    gnm_pco_dict                    = arg_list[4]
    gene_tree_ufboot_for_ale        = arg_list[5]
    genome_tree_file_subset_for_ale = arg_list[6]

    genome_tree_file_subset             = '%s_genome_tree.treefile'             % qualified_og
    gene_tree_ufboot                    = '%s.ufboot'                           % qualified_og
    gene_tree_treefile                  = '%s.treefile'                         % qualified_og
    gene_tree_treefile_subset           = '%s_subset.treefile'                  % qualified_og
    pwd_genome_tree_file_subset         = '%s/%s'                               % (gene_tree_dir, genome_tree_file_subset)
    pwd_genome_tree_file_subset_for_ale = '%s/%s'                               % (ale_wd, genome_tree_file_subset_for_ale)
    pwd_gene_tree_ufboot                = '%s/%s'                               % (gene_tree_dir, gene_tree_ufboot)
    pwd_gene_tree_ufboot_for_ale        = '%s/%s'                               % (ale_wd, gene_tree_ufboot_for_ale)
    pwd_gene_tree_treefile              = '%s/%s'                               % (gene_tree_dir, gene_tree_treefile)
    pwd_gene_tree_treefile_subset       = '%s/%s'                               % (gene_tree_dir, gene_tree_treefile_subset)

    # get genomes on gene tree
    gene_gnm_set = set()
    gnm_to_gene_dict = dict()
    for each_gene in Tree(pwd_gene_tree_treefile).get_leaf_names():

        # get gnm id
        gene_gnm = '_'.join(each_gene.split('_')[:-1])

        gene_gnm_set.add(gene_gnm)
        if gene_gnm not in gnm_to_gene_dict:
            gnm_to_gene_dict[gene_gnm] = {each_gene}
        else:
            gnm_to_gene_dict[gene_gnm].add(each_gene)

    # subset genome tree
    genome_tree_leaf_set = Tree(genome_tree_file_rooted).get_leaf_names()
    gnms_in_both_trees = set(genome_tree_leaf_set).intersection(gene_gnm_set)
    gnm_tree_subset_str = subset_tree(genome_tree_file_rooted, gnms_in_both_trees, None)
    gnm_tree_subset_str_for_ale = gnm_tree_subset_str
    gnm_tree_subset_str_for_ale = gnm_tree_subset_str_for_ale.replace('GCA_', 'GCA').replace('GCF_', 'GCF')

    # write out genome tree subset
    with open(pwd_genome_tree_file_subset, 'w') as pwd_genome_tree_file_subset_handle:
        pwd_genome_tree_file_subset_handle.write(gnm_tree_subset_str)

    # write out genome tree subset for running ALE
    with open(pwd_genome_tree_file_subset_for_ale, 'w') as pwd_genome_tree_file_subset_for_ale_handle:
        pwd_genome_tree_file_subset_for_ale_handle.write(gnm_tree_subset_str_for_ale)

    # get genes to keep in gene tree
    gene_set_to_keep = set()
    for each_gnm in gnms_in_both_trees:
        gene_set_to_keep.update(gnm_to_gene_dict.get(each_gnm, set()))

    # subset gene_tree.treefile
    subset_tree(pwd_gene_tree_treefile, gene_set_to_keep, pwd_gene_tree_treefile_subset)

    # subset gene_tree.ufboot and rename leaves for running ALE
    pwd_gene_tree_ufboot_for_ale_handle = open(pwd_gene_tree_ufboot_for_ale, 'w')
    for each_gene_tree in open(pwd_gene_tree_ufboot):
        gene_tree_str = each_gene_tree.strip()
        gene_tree_str_subset_for_ale = subset_tree(gene_tree_str, gene_set_to_keep, None)
        gene_tree_str_subset_for_ale = gene_tree_str_subset_for_ale.replace('GCA_', 'GCA').replace('GCF_', 'GCF')
        pwd_gene_tree_ufboot_for_ale_handle.write(gene_tree_str_subset_for_ale + '\n')
    pwd_gene_tree_ufboot_for_ale_handle.close()

    # get gene tree leaf name dict (for plot)
    leaf_name_dict = dict()
    for each_gene in Tree(pwd_gene_tree_treefile_subset).get_leaf_names():
        gene_id = each_gene
        gene_genome = '_'.join(gene_id.split('_')[:-1])
        genome_pco = gnm_pco_dict[gene_genome]
        gene_id_with_taxon = '%s_%s' % (genome_pco, gene_id.split('_')[-1])
        leaf_name_dict[gene_id] = gene_id_with_taxon


def ALE1(args):

    orthogroups_op_txt  = args['i']
    combined_faa        = args['s']
    og_program          = args['p']
    genome_taxon_txt    = args['c']
    min_og_genome_num   = args['m']
    min_og_phylum_num   = args['n']
    num_threads         = args['t']
    js_num_threads      = args['jt']
    force_create_op_dir = args['f']
    op_dir              = args['o']
    designate_ogs       = []
    to_ignore_ogs_list  = []

    # define output file name
    get_gene_tree_cmds_txt = '%s_cmds.txt' % op_dir

    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()

    if force_create_op_dir is True:
        if os.path.isdir(op_dir) is True:
            os.system('rm -r %s' % op_dir)
    os.system('mkdir %s' % op_dir)

    # read in genome taxonomy
    gnm_p_dict   = dict()
    gnm_c_dict   = dict()
    gnm_o_dict   = dict()
    gnm_pco_dict = dict()
    for each_gnm in open(genome_taxon_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        gnm_id     = each_gnm_split[0]
        taxon_str  = each_gnm_split[1]
        gnm_phylum = taxon_str.split(';')[1]
        gnm_class  = taxon_str.split(';')[2]
        gnm_order  = taxon_str.split(';')[3]
        gnm_p_dict[gnm_id] = gnm_phylum
        gnm_c_dict[gnm_id] = gnm_class
        gnm_o_dict[gnm_id] = gnm_order
        gnm_pco_dict[gnm_id] = '%s__%s__%s__%s' % (gnm_phylum[3:], gnm_class[3:], gnm_order[3:], gnm_id)

    # get ortho_to_gene_dict
    ortho_to_gene_dict = get_ortho_to_gene_dict(orthogroups_op_txt, og_program)

    # get qualified orthogroups
    qualified_og_set = set()
    for each_ortho in ortho_to_gene_dict:
        ortho_gene_set = ortho_to_gene_dict[each_ortho]
        ortho_p_set = set()
        ortho_gnm_set = set()
        for each_gene in ortho_gene_set:
            gene_gnm = '_'.join(each_gene.split('_')[:-1])
            gnm_taxon = gnm_p_dict[gene_gnm]
            ortho_gnm_set.add(gene_gnm)
            ortho_p_set.add(gnm_taxon)
        if (len(ortho_gnm_set) >= min_og_genome_num) and (len(ortho_p_set) >= min_og_phylum_num):
            qualified_og_set.add(each_ortho)
    print('The total number of identified orthogroups is %s.' % len(ortho_to_gene_dict))
    print('The number of orthogroups spanning >= %s genomes and >= %s phyla is %s.' % (min_og_genome_num, min_og_phylum_num, len(qualified_og_set)))

    # process qualified OG
    og_to_process = sorted([i for i in qualified_og_set])
    if len(designate_ogs) > 0:
        print('The number of designated OGs to process: %s' % len(designate_ogs))
        og_to_process = designate_ogs

    og_to_process_no_ignored = set()
    for each_og in og_to_process:
        if each_og not in to_ignore_ogs_list:
            og_to_process_no_ignored.add(each_og)

    # extract gene sequences and prepare commands for building gene tree
    print('Preparing commands for building gene trees')
    extract_seq_arg_lol = []
    prepare_ale_ip_worker_arg_lol = []
    get_gene_tree_cmds_txt_handle = open(get_gene_tree_cmds_txt, 'w')
    for qualified_og in sorted(og_to_process_no_ignored):
        qualified_og_gene_set         = ortho_to_gene_dict[qualified_og]
        qualified_og_gene_txt         = '%s/%s.txt'         % (op_dir, qualified_og)
        qualified_og_gene_faa         = '%s/%s.faa'         % (op_dir, qualified_og)
        qualified_og_gene_aln         = '%s/%s.aln'         % (op_dir, qualified_og)
        qualified_og_gene_aln_trimmed = '%s/%s_trimmed.aln' % (op_dir, qualified_og)
        pwd_gene_tree_ufboot          = '%s/%s.ufboot'      % (op_dir, qualified_og)

        # write out the id of genes
        with open(qualified_og_gene_txt, 'w') as qualified_og_gene_txt_handle:
            qualified_og_gene_txt_handle.write('\n'.join(qualified_og_gene_set))

        # add to mp lol
        extract_seq_arg_lol.append([combined_faa, qualified_og_gene_txt, qualified_og_gene_faa])

        # write out js for mafft, trimal and iqtree
        mafft_cmd           = 'mafft-einsi --thread %s --quiet %s.faa > %s.aln'                     % (js_num_threads, qualified_og, qualified_og)
        trimal_cmd          = 'trimal -in %s.aln -out %s -automated1'                               % (qualified_og, qualified_og_gene_aln_trimmed)
        iqtree_cmd          = 'iqtree -m LG+G+I -bb 1000 --wbtl -nt %s -s %s.aln -pre %s'           % (js_num_threads, qualified_og, qualified_og)
        iqtree_cmd_trimmed  = 'iqtree -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s_trimmed'       % (js_num_threads, qualified_og_gene_aln_trimmed, qualified_og)
        get_gene_tree_cmds_txt_handle.write('%s; %s\n' % (mafft_cmd, iqtree_cmd))
    get_gene_tree_cmds_txt_handle.close()

    # extract gene sequences with multiprocessing
    print('Extracting gene sequences with %s cores' % num_threads)
    pool = mp.Pool(processes=num_threads)
    pool.map(select_seq, extract_seq_arg_lol)
    pool.close()
    pool.join()


def ALE2(args):

    gene_tree_dir           = args['i']
    genome_tree_file_rooted = args['s']
    genome_taxon_txt        = args['c']
    force_create_ale_wd     = args['f']
    num_threads             = args['t']
    ale_wd                  = args['o']
    designate_ogs           = []
    to_ignore_ogs_list      = []
    run_ale_cmds_txt        = '%s_cmds.txt' % ale_wd

    ufboot_file_re   = '%s/*.ufboot' % gene_tree_dir
    ufboot_file_list = glob.glob(ufboot_file_re)

    og_to_process_list = []
    for each_ufboot in ufboot_file_list:
        _, ufboot_base, _ = sep_path_basename_ext(each_ufboot)
        og_to_process_list.append(ufboot_base)

    # read in genome taxonomy
    gnm_pco_dict = dict()
    for each_gnm in open(genome_taxon_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        gnm_id     = each_gnm_split[0]
        taxon_str  = each_gnm_split[1]
        gnm_phylum = taxon_str.split(';')[1]
        gnm_class  = taxon_str.split(';')[2]
        gnm_order  = taxon_str.split(';')[3]
        gnm_pco_dict[gnm_id] = '%s__%s__%s__%s' % (gnm_phylum[3:], gnm_class[3:], gnm_order[3:], gnm_id)

    # create ale_wd
    if force_create_ale_wd is True:
        if os.path.isdir(ale_wd) is True:
            os.system('rm -r %s' % ale_wd)
    os.system('mkdir %s' % ale_wd)

    run_ale_cmds_txt_handle = open(run_ale_cmds_txt, 'w')
    prepare_ale_ip_worker_arg_lol = []
    for qualified_og in og_to_process_list:
        pwd_gene_tree_ufboot = '%s/%s.ufboot' % (gene_tree_dir, qualified_og)
        if os.path.isfile(pwd_gene_tree_ufboot) is False:
            print('%s not found, please build gene tree first!' % pwd_gene_tree_ufboot)
        else:
            gene_tree_ufboot_for_ale        = '%s_for_ALE.ufboot'                      % qualified_og
            genome_tree_file_subset_for_ale = '%s_genome_tree_for_ALE.treefile'        % qualified_og
            obtain_ale_file_cmd             = 'ALEobserve %s'                          % gene_tree_ufboot_for_ale
            reconciliation_cmd              = 'ALEml_undated %s %s_for_ALE.ufboot.ale' % (genome_tree_file_subset_for_ale, qualified_og)
            current_arg_list                = [qualified_og, gene_tree_dir, ale_wd, genome_tree_file_rooted, gnm_pco_dict, gene_tree_ufboot_for_ale, genome_tree_file_subset_for_ale]
            run_ale_cmds_txt_handle.write('%s; %s\n' % (obtain_ale_file_cmd, reconciliation_cmd))
            prepare_ale_ip_worker_arg_lol.append(current_arg_list)
    run_ale_cmds_txt_handle.close()

    # prepare input files and job script for running ALE with multiprocessing
    print('Preparing files for running ALE with %s cores for %s OGs' % (num_threads, len(prepare_ale_ip_worker_arg_lol)))
    pool = mp.Pool(processes=num_threads)
    pool.map(prepare_ale_ip_worker, prepare_ale_ip_worker_arg_lol)
    pool.close()
    pool.join()


def iTOL(Leaf_to_Group_dict, Group_to_Color_dict, FileOut):

    Group_set = set()
    for each_leaf in Leaf_to_Group_dict:
        Group_set.add(Leaf_to_Group_dict[each_leaf])

    if len(Group_to_Color_dict) == 0:
        Group_to_Color_dict = dict(zip(Group_set, get_color_list(len(Group_set))))
    else:
        group_without_color_list = []
        for each_group in Group_set:
            if each_group not in Group_to_Color_dict:
                group_without_color_list.append(each_group)
        if len(group_without_color_list) > 0:
            color_list_unprovided = get_color_list(len(group_without_color_list))
            Group_to_Color_dict_unprovided = dict(zip(group_without_color_list, color_list_unprovided))
            for each_group in Group_to_Color_dict_unprovided:
                Group_to_Color_dict[each_group] = Group_to_Color_dict_unprovided[each_group]

    FileOut_handle = open(FileOut, 'w')
    FileOut_handle.write('DATASET_COLORSTRIP\n')
    FileOut_handle.write('SEPARATOR TAB\n')
    FileOut_handle.write('DATASET_LABEL\tTaxonomy\n')
    FileOut_handle.write('\n# customize strip attributes here\n')
    FileOut_handle.write('STRIP_WIDTH\t100\n')
    FileOut_handle.write('MARGIN\t20\n')
    FileOut_handle.write('\n# provide data here\nDATA\n')
    for leaf in Leaf_to_Group_dict:
        leaf_group = Leaf_to_Group_dict[leaf]
        leaf_color = Group_to_Color_dict[leaf_group]
        FileOut_handle.write('%s\t%s\t%s\n' % (leaf, leaf_color, leaf_group))
    FileOut_handle.close()


def parse_ale_op_worker(arg_list):

    qualified_og                = arg_list[0]
    gene_tree_dir               = arg_list[1]
    ale_wd                      = arg_list[2]
    ale_op_dir                  = arg_list[3]
    ale_hgt_plot_dir            = arg_list[4]
    interal_node_prefix         = arg_list[5]
    gnm_pco_dict                = arg_list[6]
    d_color                     = arg_list[7]
    r_color                     = arg_list[8]
    project_name                = arg_list[9]
    API_key                     = arg_list[10]
    display_mode                = arg_list[11]
    hgt_freq_cutoff             = arg_list[12]
    ignore_leaf_hgt             = arg_list[13]
    ignore_vertical_hgt         = arg_list[14]
    donor_node_min_leaf_num     = arg_list[15]
    recipient_node_min_leaf_num = arg_list[16]
    dr_separator                = arg_list[17]
    root_gene_tree_at_midpoint  = arg_list[18]
    p_color_txt                 = arg_list[19]

    ale_uml_rec_file                                    = '%s/%s_for_ALE.ufboot.ale.uml_rec'                    % (ale_wd, qualified_og)
    gene_tree_treefile                                  = '%s.treefile'                                         % qualified_og
    genome_tree_file_subset_for_ale                     = '%s_genome_tree_for_ALE.treefile'                     % qualified_og
    gene_tree_ufboot_for_ale                            = '%s_for_ALE.ufboot'                                   % qualified_og
    uts_file                                            = '%s.ale.uTs'                                          % gene_tree_ufboot_for_ale
    uml_rec_file                                        = '%s.ale.uml_rec'                                      % gene_tree_ufboot_for_ale
    ale_formatted_gnm_tree                              = '%s_ALE_formatted_genome_tree.tree'                   % gene_tree_ufboot_for_ale
    ale_formatted_gnm_tree_with_len                     = '%s_ALE_formatted_genome_tree_with_len.tree'          % gene_tree_ufboot_for_ale
    ale_formatted_gnm_tree_with_len_prefixed            = '%s_ALE_formatted_genome_tree_with_len_prefixed.tree' % gene_tree_ufboot_for_ale
    itol_connection_txt_all                             = '%s_iTOL_connection.txt'                              % qualified_og
    itol_label_txt                                      = '%s_iTOL_genome_pco.txt'                              % qualified_og
    gene_tree_itol_label_txt                            = '%s_iTOL_gene_pco.txt'                                % qualified_og
    gene_tree_treefile_subset                           = '%s_subset.treefile'                                  % qualified_og
    gene_tree_treefile_subset_midpoint_rooted           = '%s_subset_midpoint_rooted.treefile'                  % qualified_og
    gene_tree_itol_colorstrip_txt                       = '%s_iTOL_colorstrip_gene.txt'                         % qualified_og
    genome_tree_itol_colorstrip_txt                     = '%s_iTOL_colorstrip_genome.txt'                       % qualified_og
    pwd_gene_tree_treefile_subset                       = '%s/%s'                                               % (gene_tree_dir, gene_tree_treefile_subset)
    pwd_gene_tree_treefile_subset_midpoint_rooted       = '%s/%s'                                               % (ale_op_dir, gene_tree_treefile_subset_midpoint_rooted)
    pwd_gene_tree_treefile                              = '%s/%s'                                               % (gene_tree_dir, gene_tree_treefile)
    pwd_genome_tree_file_subset_for_ale                 = '%s/%s'                                               % (ale_op_dir, genome_tree_file_subset_for_ale)
    pwd_itol_connection_txt_all                         = '%s/%s'                                               % (ale_hgt_plot_dir, itol_connection_txt_all)
    pwd_itol_label_txt                                  = '%s/%s'                                               % (ale_op_dir, itol_label_txt)
    pwd_gene_tree_itol_label_txt                        = '%s/%s'                                               % (ale_hgt_plot_dir, gene_tree_itol_label_txt)
    pwd_uts_file                                        = '%s/%s'                                               % (ale_op_dir, uts_file)
    pwd_uml_rec_file                                    = '%s/%s'                                               % (ale_op_dir, uml_rec_file)
    pwd_ale_formatted_gnm_tree                          = '%s/%s'                                               % (ale_op_dir, ale_formatted_gnm_tree)
    pwd_ale_formatted_gnm_tree_with_len                 = '%s/%s'                                               % (ale_op_dir, ale_formatted_gnm_tree_with_len)
    pwd_ale_formatted_gnm_tree_with_len_prefixed        = '%s/%s'                                               % (ale_op_dir, ale_formatted_gnm_tree_with_len_prefixed)
    pwd_gene_tree_itol_colorstrip_txt                   = '%s/%s'                                               % (ale_hgt_plot_dir, gene_tree_itol_colorstrip_txt)
    pwd_genome_tree_itol_colorstrip_txt                 = '%s/%s'                                               % (ale_hgt_plot_dir, genome_tree_itol_colorstrip_txt)

    # run ale_splitter
    ale_splitter(ale_uml_rec_file)

    # read in phylum color
    p_color_dict = dict()
    for each_line in open(p_color_txt):
        each_line_split = each_line.strip().split('\t')
        phylum_id = each_line_split[1]
        color_id = each_line_split[0]
        p_color_dict[phylum_id] = color_id

    internal_node_to_leaf_dict = dict()
    paired_donor_to_recipient_leaf_dict = dict()
    hgt_freq_dict = dict()
    if os.path.isfile(pwd_uts_file) is True:

        # write out ALE formatted genome tree
        renamed_genome_tree_str = open(pwd_uml_rec_file).readlines()[2].strip().split('\t')[1]
        with open(pwd_ale_formatted_gnm_tree, 'w') as ale_renamed_species_tree_handle:
            ale_renamed_species_tree_handle.write(renamed_genome_tree_str + '\n')

        internal_node_to_leaf_dict, paired_donor_to_recipient_leaf_dict, hgt_freq_dict = uts_to_itol_connections(pwd_genome_tree_file_subset_for_ale, pwd_ale_formatted_gnm_tree, interal_node_prefix, pwd_uts_file, hgt_freq_cutoff, ignore_leaf_hgt, ignore_vertical_hgt, donor_node_min_leaf_num, recipient_node_min_leaf_num, pwd_itol_connection_txt_all, dr_separator)
    else:
        print('%s: uTs file not found, you need to run ALE first!' % qualified_og)

    # combine_trees
    combine_trees(pwd_genome_tree_file_subset_for_ale, pwd_ale_formatted_gnm_tree, pwd_ale_formatted_gnm_tree_with_len)

    # prefix_internal_nodes of combined tree
    prefix_internal_nodes(pwd_ale_formatted_gnm_tree_with_len, interal_node_prefix, pwd_ale_formatted_gnm_tree_with_len_prefixed)

    # write out iTOL label file for gene and genome tree, also colorstrip for taxonomy
    pwd_itol_label_txt_handle  = open(pwd_itol_label_txt, 'w')
    pwd_itol_label_txt_handle.write('LABELS\nSEPARATOR TAB\n\nDATA\n')
    pwd_gene_tree_itol_label_txt_handle = open(pwd_gene_tree_itol_label_txt, 'w')
    pwd_gene_tree_itol_label_txt_handle.write('LABELS\nSEPARATOR TAB\n\nDATA\n')
    wrote_gnm_set = set()
    gene_to_p_dict = dict()
    genome_to_p_dict = dict()
    for each_gene in Tree(pwd_gene_tree_treefile).get_leaf_names():
        gene_gnm = '_'.join(each_gene.split('_')[:-1])
        genome_name_for_ale = gene_gnm
        genome_name_for_ale = genome_name_for_ale.replace('GCA_', 'GCA').replace('GCF_', 'GCF')
        genome_with_taxon = gnm_pco_dict[gene_gnm]
        gene_to_p_dict[each_gene] = genome_with_taxon.split('__')[0]
        if gene_gnm not in wrote_gnm_set:
            genome_to_p_dict[genome_name_for_ale] = genome_with_taxon.split('__')[0]
            pwd_itol_label_txt_handle.write('%s\t%s\n' % (genome_name_for_ale, genome_with_taxon))
            wrote_gnm_set.add(gene_gnm)
        pwd_gene_tree_itol_label_txt_handle.write('%s\t%s_%s\n' % (each_gene, genome_with_taxon, each_gene.split('_')[-1]))
    pwd_itol_label_txt_handle.close()
    pwd_gene_tree_itol_label_txt_handle.close()

    iTOL(gene_to_p_dict, p_color_dict, pwd_gene_tree_itol_colorstrip_txt)
    iTOL(genome_to_p_dict, p_color_dict, pwd_genome_tree_itol_colorstrip_txt)

    # root gene tree at midpoint
    gene_tree_to_plot = pwd_gene_tree_treefile_subset
    if root_gene_tree_at_midpoint is True:
        root_at_midpoint(pwd_gene_tree_treefile_subset, pwd_gene_tree_treefile_subset_midpoint_rooted)
        gene_tree_to_plot = pwd_gene_tree_treefile_subset_midpoint_rooted

    # plot separately
    n = 1
    for each_d2r in paired_donor_to_recipient_leaf_dict:
        each_d2r_freq = hgt_freq_dict[each_d2r]
        each_d2r_d_list = paired_donor_to_recipient_leaf_dict[each_d2r][0]
        each_d2r_r_list = paired_donor_to_recipient_leaf_dict[each_d2r][1]
        pwd_itol_label_txt                                  = '%s/%s_iTOL_genome_pco.txt'               % (ale_op_dir, qualified_og)
        pwd_gene_tree_itol_label_txt                        = '%s/%s_iTOL_gene_pco.txt'                 % (ale_hgt_plot_dir, qualified_og)
        pwd_gnm_tree_label_color_txt                        = '%s/%s_iTOL_label_color_genome_%s.txt'    % (ale_hgt_plot_dir, qualified_og, each_d2r)
        pwd_gene_tree_label_color_txt                       = '%s/%s_iTOL_label_color_gene_%s.txt'      % (ale_hgt_plot_dir, qualified_og, each_d2r)
        pwd_itol_connection_txt                             = '%s/%s_iTOL_connection_%s.txt'            % (ale_hgt_plot_dir, qualified_og, each_d2r)
        pwd_ale_formatted_gnm_tree_with_len_prefixed_pdf    = '%s/%s_genome_tree_with_HGT_%s.pdf'       % (ale_wd, qualified_og, each_d2r)
        pwd_gene_tree_treefile_subset_pdf                   = '%s/%s_subset_%s.pdf'                     % (ale_hgt_plot_dir, qualified_og, each_d2r)
        pwd_gene_tree_treefile_subset_pdf_rooted            = '%s/%s_subset_%s_rooted.pdf'              % (ale_hgt_plot_dir, qualified_og, each_d2r)
        pwd_combined_image_with_ale_hgts                    = '%s/%s_HGT_%s_%s_%s.pdf'                  % (ale_hgt_plot_dir, qualified_og, n, each_d2r, each_d2r_freq)

        # write out gnm_tree_label_color_txt
        pwd_gnm_tree_label_color_txt_handle = open(pwd_gnm_tree_label_color_txt, 'w')
        pwd_gnm_tree_label_color_txt_handle.write('DATASET_STYLE\nSEPARATOR TAB\nDATASET_LABEL\texample_style\nCOLOR\t#ffff00\n\nDATA\n')
        pwd_gnm_tree_label_color_txt_handle.write('%s\tlabel\tclade\t%s\t1\tnormal\n' % (each_d2r.split(dr_separator)[0], d_color))
        pwd_gnm_tree_label_color_txt_handle.write('%s\tlabel\tclade\t%s\t1\tnormal\n' % (each_d2r.split(dr_separator)[1], r_color))
        pwd_gnm_tree_label_color_txt_handle.close()

        # write out iTOL label file for gene and genome tree, also colorstrip for taxonomy
        pwd_gene_tree_label_color_txt_handle = open(pwd_gene_tree_label_color_txt, 'w')
        pwd_gene_tree_label_color_txt_handle.write('DATASET_STYLE\nSEPARATOR TAB\nDATASET_LABEL\texample_style\nCOLOR\t#ffff00\n\nDATA\n')
        for each_gene in Tree(pwd_gene_tree_treefile).get_leaf_names():

            gene_name_for_ale = '_'.join(each_gene.strip().split('_')[:-1])
            gene_name_for_ale = gene_name_for_ale.replace('GCA_', 'GCA').replace('GCF_', 'GCF')
            if gene_name_for_ale in each_d2r_d_list:
                pwd_gene_tree_label_color_txt_handle.write('%s\tlabel\tnode\t%s\t1\tnormal\n' % (each_gene, d_color))
            elif gene_name_for_ale in each_d2r_r_list:
                pwd_gene_tree_label_color_txt_handle.write('%s\tlabel\tnode\t%s\t1\tnormal\n' % (each_gene, r_color))
        pwd_gene_tree_label_color_txt_handle.close()

        itol_tree(pwd_ale_formatted_gnm_tree_with_len_prefixed, [pwd_gnm_tree_label_color_txt, pwd_itol_label_txt, pwd_itol_connection_txt, pwd_genome_tree_itol_colorstrip_txt], project_name, API_key, display_mode, pwd_ale_formatted_gnm_tree_with_len_prefixed_pdf)
        itol_tree(gene_tree_to_plot, [pwd_gene_tree_itol_label_txt, pwd_gene_tree_label_color_txt, pwd_gene_tree_itol_colorstrip_txt], project_name, API_key, display_mode, pwd_gene_tree_treefile_subset_pdf)
        merge_pdf(pwd_ale_formatted_gnm_tree_with_len_prefixed_pdf, pwd_gene_tree_treefile_subset_pdf, 66, pwd_combined_image_with_ale_hgts)
        n += 1

        os.system('mv %s %s/annotation_files/' % (pwd_ale_formatted_gnm_tree_with_len_prefixed_pdf, ale_hgt_plot_dir))
        os.system('mv %s %s/annotation_files/' % (pwd_gene_tree_treefile_subset_pdf, ale_hgt_plot_dir))
        os.system('mv %s %s/annotation_files/' % (pwd_gnm_tree_label_color_txt, ale_hgt_plot_dir))
        os.system('mv %s %s/annotation_files/' % (pwd_gene_tree_label_color_txt, ale_hgt_plot_dir))
        os.system('mv %s %s/annotation_files/' % (pwd_itol_connection_txt, ale_hgt_plot_dir))
    os.system('mv %s %s/annotation_files/' % (pwd_itol_label_txt, ale_hgt_plot_dir))
    os.system('mv %s %s/annotation_files/' % (pwd_gene_tree_itol_label_txt, ale_hgt_plot_dir))
    os.system('mv %s %s/annotation_files/' % (pwd_itol_connection_txt_all, ale_hgt_plot_dir))
    os.system('mv %s %s/annotation_files/' % (pwd_gene_tree_itol_colorstrip_txt, ale_hgt_plot_dir))
    os.system('mv %s %s/annotation_files/' % (pwd_genome_tree_itol_colorstrip_txt, ale_hgt_plot_dir))


def ale_splitter(rec_file):

    options = [True, True, True, True]
    with open(rec_file) as f:
        lines               = f.readlines()
        stree               = lines[2].strip()
        ll                  = lines[6].strip().split()[-1]
        rates               = lines[8].strip().split("\t")[1:]
        n_reconciled_trees  = int(lines[10].strip().split()[0])
        reconciled_trees    = lines[12:n_reconciled_trees + 12]
        n_of_events         = lines[12 + n_reconciled_trees + 1].split("\t")[1:]
        table               = lines[12 + n_reconciled_trees + 3:]

    if options[0]:
        with open(rec_file.replace("uml_rec", "stree"), "w") as f:
            f.write(stree.split("\t")[-1])
    if options[1]:
        with open(rec_file.replace("uml_rec", "info"), "w") as f:
            f.write("LL:" + "\t" + ll + "\n")
            f.write("Dp:" + "\t" + rates[0] + "\n")
            f.write("Tp:" + "\t" + rates[1] + "\n")
            f.write("Lp:" + "\t" + rates[2] + "\n")
            f.write("De:" + "\t" + n_of_events[0] + "\n")
            f.write("Te:" + "\t" + n_of_events[1] + "\n")
            f.write("Le:" + "\t" + n_of_events[2] + "\n")
            f.write("Se:" + "\t" + n_of_events[3] + "\n")
    if options[2]:
        with open(rec_file.replace("uml_rec", "recs"), "w") as f:
            for t in reconciled_trees:
                f.write(t)
    if options[3]:
        with open(rec_file.replace("uml_rec", "rec_table"), "w") as f:
            for e in table:
                f.write(e)


def ALE3(args):

    gene_tree_dir               = args['i1']
    ale_wd                      = args['i2']
    genome_taxon_txt            = args['c']
    ar_phylum_color_code_txt    = args['color']
    ale_hgt_plot_dir            = args['o']
    force_create_op_dir         = args['f']
    API_key                     = args['api']
    hgt_freq_cutoff             = args['fc']
    donor_node_min_leaf_num     = args['mld']
    recipient_node_min_leaf_num = args['mlr']
    project_name                = args['itol']

    ignore_vertical_hgt             = True                      # filter ALE predicted HGTs
    ignore_leaf_hgt                 = True                      # filter ALE predicted HGTs
    interal_node_prefix             = 'IN'                      # plot tree with HGT
    display_mode                    = '1'                       # plot tree with HGT, 1=rectangular, 2=circular, 3=unrooted
    align_leaf_name                 = True                      # plot tree with HGT
    show_scale                      = False                     # plot tree with HGT
    d_color                         = '#FF0000'                 # plot tree with HGT
    r_color                         = '#0000FF'                 # plot tree with HGT
    dr_separator                    = '_to_'                    # plot tree with HGT
    root_gene_tree_at_midpoint      = True                      # plot tree with HGT

    ####################################################################################################################

    ufboot_file_re   = '%s/*.ufboot' % gene_tree_dir
    ufboot_file_list = glob.glob(ufboot_file_re)
    og_to_process_list = []
    for each_ufboot in ufboot_file_list:
        _, ufboot_base, _ = sep_path_basename_ext(each_ufboot)
        og_to_process_list.append(ufboot_base)

    # read in genome taxonomy
    gnm_pco_dict = dict()
    for each_gnm in open(genome_taxon_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        gnm_id     = each_gnm_split[0]
        taxon_str  = each_gnm_split[1]
        gnm_phylum = taxon_str.split(';')[1]
        gnm_class  = taxon_str.split(';')[2]
        gnm_order  = taxon_str.split(';')[3]
        gnm_pco_dict[gnm_id] = '%s__%s__%s__%s' % (gnm_phylum[3:], gnm_class[3:], gnm_order[3:], gnm_id)

    if os.path.isdir(ale_hgt_plot_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % ale_hgt_plot_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % ale_hgt_plot_dir)
    os.system('mkdir %s/annotation_files' % ale_hgt_plot_dir)

    # parse ALE output
    n = 1
    for qualified_og in og_to_process_list:

        print('%s (%s/%s): Parsing ALE outputs' % (qualified_og, n, len(og_to_process_list)))
        current_arg_list = [qualified_og, gene_tree_dir, ale_wd, ale_wd, ale_hgt_plot_dir, interal_node_prefix,
                            gnm_pco_dict, d_color, r_color, project_name, API_key, display_mode, hgt_freq_cutoff,
                            ignore_leaf_hgt, ignore_vertical_hgt, donor_node_min_leaf_num, recipient_node_min_leaf_num,
                            dr_separator, root_gene_tree_at_midpoint, ar_phylum_color_code_txt]
        parse_ale_op_worker(current_arg_list)
        n += 1

    print('Done!')


def ALE4(args):
    pass


if __name__ == '__main__':
    pass

    # ALE1_parser = argparse.ArgumentParser()
    # ALE1_parser.add_argument('-i',   required=True,                         help='orthologous groups, either from orthofinder or oma')
    # ALE1_parser.add_argument('-s',   required=True,                         help='sequence file, e.g., combined.faa')
    # ALE1_parser.add_argument('-p',   required=True,                         help='orthologous identification program, orthofinder or oma')
    # ALE1_parser.add_argument('-m',   required=False, type=int, default=50,  help='min_og_genome_num, default: 50')
    # ALE1_parser.add_argument('-n',   required=False, type=int, default=2,   help='min_og_phylum_num, default: 2')
    # ALE1_parser.add_argument('-o',   required=True,                         help='output dir, i.e., OMA working directory')
    # ALE1_parser.add_argument('-t',   required=False, type=int, default=6,   help='number of threads, default: 6')
    # ALE1_parser.add_argument('-jt',  required=False, type=int, default=3,   help='number of threads for job script, default: 3')
    # ALE1_parser.add_argument('-f',   required=False, action="store_true",   help='force overwrite')
    # ALE1_parser.add_argument('-c',   required=True,                         help='genome_taxon_txt')
    # args = vars(ALE1_parser.parse_args())
    # ALE1(args)

    # ALE2_parser = argparse.ArgumentParser()
    # ALE2_parser.add_argument('-i',   required=True,                         help='ALE1 output directory')
    # ALE2_parser.add_argument('-s',   required=True,                         help='rooted species tree')
    # ALE2_parser.add_argument('-c',   required=True,                         help='genome_taxon_txt')
    # ALE2_parser.add_argument('-o',   required=True,                         help='output dir, i.e., OMA working directory')
    # ALE2_parser.add_argument('-f',   required=False, action="store_true",   help='force overwrite')
    # ALE2_parser.add_argument('-t',   required=False, type=int, default=6,   help='number of threads, default: 6')
    # args = vars(ALE2_parser.parse_args())
    # ALE2(args)

    # ALE3_parser = argparse.ArgumentParser()
    # ALE3_parser.add_argument('-i1',     required=True,                          help='ALE1 output directory')
    # ALE3_parser.add_argument('-i2',     required=True,                          help='ALE2 output directory')
    # ALE3_parser.add_argument('-c',      required=True,                          help='genome_taxon_txt')
    # ALE3_parser.add_argument('-color',  required=True,                          help='phylum_color_code.txt')
    # ALE3_parser.add_argument('-o',      required=True,                          help='output dir, i.e., ALE3_op_dir')
    # ALE3_parser.add_argument('-f',      required=False, action="store_true",    help='force overwrite')
    # args = vars(ALE3_parser.parse_args())
    # ALE3(args)



import operator


def keep_highest_rrtc(rrtc_in, rrtc_out):

    rrtc_highest_prob_dict = dict()
    for each_rrtc in open(rrtc_in):
        each_rrtc_split = each_rrtc.strip().split(':')[0].split('\t')
        rrtc_r = each_rrtc.strip().split(':')[0].split('\t')[0]
        rrtc_d = each_rrtc.strip().split(':')[0].split('\t')[1]
        rrtc_v = float(each_rrtc.strip().split(':')[1])
        rrtc_key = '%s___%s' % (rrtc_r, rrtc_d)
        if rrtc_key not in rrtc_highest_prob_dict:
            rrtc_highest_prob_dict[rrtc_key] = rrtc_v
        else:
            if rrtc_v > rrtc_highest_prob_dict[rrtc_key]:
                rrtc_highest_prob_dict[rrtc_key] = rrtc_v

    with open(rrtc_out, 'w') as rrtc_out_handle:
        for each_rrtc in sorted(rrtc_highest_prob_dict.items(), key=operator.itemgetter(1))[::-1]:
            print(each_rrtc)
            rrtc_r = each_rrtc[0].split('___')[0]
            rrtc_d = each_rrtc[0].split('___')[1]
            rrtc_v = each_rrtc[1]
            rrtc_out_handle.write('%s\t%s:%s\n' % (rrtc_r, rrtc_d, rrtc_v))


rrtc_in  = '/Users/songweizhi/Desktop/rrtc.txt'
rrtc_out = '/Users/songweizhi/Desktop/rrtc_out.txt'
keep_highest_rrtc(rrtc_in, rrtc_out)


demo_dict = { 'a': 6, 'b': 2, 'c': 2 }
for each in sorted(demo_dict.items(), key=operator.itemgetter(1))[::-1]:
    print(each[0])
    print(each[1])




import os
import glob
import argparse
import pandas as pd
from ete3 import Tree
import plotly.express as px


PlotMcmcNode_usage = '''
============================ PlotMcmcNode example commands ============================

TreeSAK PlotMcmcNode -i McmcTree_op_files -n n179 -o Clock2_n179.pdf
TreeSAK PlotMcmcNode -i McmcTree_op_files -n n161,n186 -o Clock3_n161_n186.pdf
TreeSAK PlotMcmcNode -i McmcTree_op_files -n nodes.txt -o multi_runs_multi_nodes.pdf

# File name of the mcmc.txt and the corresponding mcmc out file need to follow 
# the rule as specified below:
[setting_1]_mcmc.txt
[setting_1]_out.txt
[setting_2]_mcmc.txt
[setting_2]_out.txt

# file format (-n, tab separated)
# leave the 2nd column blank for nodes without renaming
setting_1    node1	Bacteria
setting_2    node2
setting_3    node3,node9	Archaea

# Y-axis label file format (-l, tab separated)
PA_75_DeltaLL_50_clock3_mcmc.txt	DeltaLL_50
PA_75_DeltaLL_75_clock3_mcmc.txt	DeltaLL_75

=======================================================================================
'''

def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def mcmctree_out_to_tree_str(mamctree_out):

    # get tree string from mcmctree_out
    tree_str = ''
    tree_line = 0
    current_line = 1
    for each_line in open(mamctree_out):
        if 'Species tree for FigTree.  Branch lengths = posterior mean times; 95% CIs = labels' in each_line:
            tree_line = current_line + 1
        if tree_line == current_line:
            tree_str = each_line.strip()
        current_line += 1

    tree_str_no_space = tree_str.replace(' ', '')

    # rename tree nodes
    t = Tree(tree_str_no_space, format=1)
    for each_node in t.traverse():
        if each_node.is_leaf():
            node_name_new = '_'.join(each_node.name.split('_')[1:])
        else:
            node_name_new = 't_n%s' % each_node.name
        each_node.name = node_name_new

    tree_str_renamed = t.write(format=8)

    return tree_str_renamed


def plot_distribution(df_txt, output_plot):

    df = pd.read_table(df_txt, sep=',')
    run_id_list = df['Setting'].unique()
    node_id_list = df['Node'].unique()

    # sort dataframe by run id
    df = df.sort_values(by='Setting', ascending=False)

    plot_width  = 900
    plot_height = len(run_id_list)*100
    if plot_height < 360:
        plot_height = 360

    fig = px.violin(df, x="Value", y="Setting", color="Node", points=False, orientation="h", width=plot_width, height=plot_height)
    if len(node_id_list) == 1:
        fig.update_traces(side="positive", fillcolor='lightblue', width=1.6, opacity=0.75)
    else:
        fig.update_traces(side="positive", fillcolor='rgba(0,0,0,0)', width=1.6)

    fig.update_traces(showlegend=True)
    fig.layout.template = "simple_white"
    # fig.layout.width = 700
    # fig.layout.height = 750
    # fig.update_xaxes(range=[40, 0])
    # fig.update_layout(margin_t=10, title_text='Demo', title_x=0.5)
    fig.write_image(output_plot)


def get_internal_node_to_plot(node_txt, mo_file):

    tree_str = ''
    if os.path.isfile(mo_file):
        tree_str = mcmctree_out_to_tree_str(mo_file)

    # get nodes to plot
    node_set = set()
    node_rename_dict = dict()
    if os.path.isfile(node_txt) is True:
        for each in open(node_txt):
            each_split = each.strip().split('\t')
            node_str = each_split[0]

            # get internal_node_to_plot
            internal_node_to_plot = ''
            if ',' not in node_str:
                internal_node_to_plot = each_split[0]
            else:
                leaf_list = node_str.split(',')
                if tree_str == '':
                    print('*out.txt file not found, program exited!')
                    exit()
                current_lca = Tree(tree_str, format=1).get_common_ancestor(leaf_list)
                internal_node_to_plot = current_lca.name

            # add internal_node_to_plot to  node_set
            if internal_node_to_plot != '':
                node_set.add(internal_node_to_plot)

            # read in name to show in plot
            if len(each_split) == 2:
                if each_split[1] != '':
                    node_rename_dict[internal_node_to_plot] = each_split[1]
    else:
        node_set = node_txt.split(',')

    return node_set, node_rename_dict, tree_str


def PlotMcmcNode(args):

    mcmc_in             = args['i']
    node_txt            = args['n']
    output_plot         = args['o']
    specified_out_file  = args['of']
    y_label_txt         = args['l']
    keep_tmp_file       = args['tmp']

    # check MCMCTree output file/dir
    if os.path.isfile(mcmc_in) is True:
        mcmc_file_list = [mcmc_in]
    else:
        mcmc_file_re = '%s/*_mcmc.txt' % (mcmc_in)
        mcmc_file_list = glob.glob(mcmc_file_re)

    if len(mcmc_file_list) == 0:
        print('*mcmc.txt file not found, program exited!')
        exit()

    if specified_out_file is None:
        missed_out_file_list = []
        for each_mcmc_file in mcmc_file_list:
            pwd_out_file = each_mcmc_file.replace('_mcmc.txt', '_out.txt')
            if os.path.isfile(pwd_out_file) is False:
                missed_out_file_list.append(pwd_out_file)
        if len(missed_out_file_list) > 0:
            print('The following *out.txt files are missing, program exited!')
            print('\n'.join(sorted(missed_out_file_list)))
            exit()

    # read in y-axis label file
    y_label_dict = dict()
    if y_label_txt is not None:
        for each_sample in open(y_label_txt):
            each_sample_split = each_sample.strip().split('\t')
            if len(each_sample_split) == 2:
                y_label_dict[each_sample_split[0]] = each_sample_split[1]
            else:
                print('Format error: %s' % y_label_txt)
                exit()

    _, f_base, _ = sep_path_basename_ext(output_plot)
    found_matched_node = False
    op_tree_tmp  = '%s_tree.txt'  % f_base
    op_df_tmp    = '%s_data.txt'  % f_base
    op_label_tmp = '%s_label.txt' % f_base

    op_label_tmp_handle = open(op_label_tmp, 'w')
    op_tree_tmp_handle = open(op_tree_tmp, 'w')
    op_df_tmp_handle = open(op_df_tmp, 'w')
    op_df_tmp_handle.write('Value,Node,Setting\n')
    for mcmc_file in mcmc_file_list:

        mcmc_file_no_path = mcmc_file
        if '/' in mcmc_file_no_path:
            mcmc_file_no_path = mcmc_file_no_path.split('/')[-1]

        if specified_out_file is None:
            pwd_current_run_mcmc_out = mcmc_file.replace('_mcmc.txt', '_out.txt')
        else:
            pwd_current_run_mcmc_out = specified_out_file
        node_set, node_rename_dict, tree_str = get_internal_node_to_plot(node_txt, pwd_current_run_mcmc_out)
        op_tree_tmp_handle.write('%s\t%s\n' % (mcmc_file_no_path.replace('_mcmc.txt', ''), tree_str))
        label_to_write = y_label_dict.get(mcmc_file_no_path, mcmc_file_no_path)
        mcmc_df = pd.read_table(mcmc_file, index_col=0)
        for each_col in mcmc_df:
            if each_col in node_set:
                node_name_to_write = node_rename_dict.get(each_col, each_col)
                found_matched_node = True
                value_list = mcmc_df[each_col].values
                for each_value in value_list:
                    op_df_tmp_handle.write('%s,%s,%s\n' % (each_value, node_name_to_write, label_to_write))

                op_label_tmp_handle.write('%s\t%s\t%s\n' % (label_to_write, each_col, node_name_to_write))
    op_df_tmp_handle.close()
    op_label_tmp_handle.close()
    op_tree_tmp_handle.close()

    if found_matched_node is False:
        print('Provided node(s) not found, program exited!')
        exit()

    # plot distribution
    plot_distribution(op_df_tmp, output_plot)

    # remove tmp files
    if keep_tmp_file is False:
        os.system('rm %s' % op_tree_tmp)
        os.system('rm %s' % op_df_tmp)
        os.system('rm %s' % op_label_tmp)

    print('Plot exported to %s, done!' % output_plot)


if __name__ == '__main__':

    PlotMcmcNode_parser = argparse.ArgumentParser()
    PlotMcmcNode_parser.add_argument('-i',      required=True,                       help='folder holds the *mcmc.txt and *out.txt files')
    PlotMcmcNode_parser.add_argument('-of',     required=False, default=None,        help='the *out.txt file')
    PlotMcmcNode_parser.add_argument('-n',      required=True,                       help='Nodes to plot')
    PlotMcmcNode_parser.add_argument('-l',      required=False, default=None,        help='labels on y axis')
    PlotMcmcNode_parser.add_argument('-o',      required=True,                       help='Output plot')
    PlotMcmcNode_parser.add_argument('-tmp',    required=False, action="store_true", help='keep tmp files')
    args = vars(PlotMcmcNode_parser.parse_args())
    PlotMcmcNode(args)



import os
import argparse


pruneMSA_usage = '''
==================== pruneMSA example commands ====================

# Dependencies: perl and alignment_pruner.pl

TreeSAK pruneMSA -i input_msa.fasta -c 10
TreeSAK pruneMSA -i input_msa.fasta -c 5,10,20,30,40

Note:
1. This is a wrapper for alignment_pruner.pl (--chi2_prune mode) 
2. For details: https://doi.org/10.1038/s41467-020-17408-w

===================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def pruneMSA(args):

    msa_in              = args['i']
    conserved_cutoffs   = args['c']

    _, msa_path, msa_base, msa_ext = sep_path_basename_ext(msa_in)

    current_file_path   = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    alignment_pruner_pl = '%s/alignment_pruner.pl'  % current_file_path
    cutoff_list         = conserved_cutoffs.split(',')

    op_file_list = []
    for each_cutoff in cutoff_list:
        cutoff_formatted = str(float(each_cutoff)/100)
        current_msa_out     = '%s/%s_chi2p%s.%s'                        % (msa_path, msa_base, each_cutoff, msa_ext)
        perl_cmd            = 'perl %s --file %s --chi2_prune f%s > %s' % (alignment_pruner_pl,   msa_in, cutoff_formatted, current_msa_out)
        perl_cmd_to_print   = 'perl %s --file %s --chi2_prune f%s > %s' % ('alignment_pruner.pl', msa_in, cutoff_formatted, current_msa_out)
        op_file_list.append(current_msa_out)
        print(perl_cmd_to_print)
        os.system(perl_cmd)

    # report
    print('Pruned MSA exported to:')
    print('\n'.join(op_file_list))


if __name__ == '__main__':

    pruneMSA_parser = argparse.ArgumentParser()
    pruneMSA_parser.add_argument('-i', required=True, help='input MSA file')
    pruneMSA_parser.add_argument('-c', required=True, help='conservation cutoffs, comma separated')
    args = vars(pruneMSA_parser.parse_args())
    pruneMSA(args)
